

#         $Id: Concrete.tcl,v 1.5 2003/03/27 18:38:36 ramsan Exp $        

proc GiveUnitName { system magnitude } {

    switch $system {
	"N-m" { return $magnitude }
	"N-mm" {
	    switch $magnitude {
		N { return N }
		m { return mm }
		N/m { return N/mm }
		Nm { return Nmm }
		Nm/m { return Nmm/mm }
		Pa { return N/mm2 }
	    }
	}
	"Kp-cm" {
	    switch $magnitude {
		N { return kp }
		m { return cm }
		N/m { return kp/cm }
		Nm { return kpcm }
		Nm/m { return kpcm/cm }
		Pa { return kp/cm2 }
	    }
	}
	"kN-m-MPa" {
	    switch $magnitude {
		N { return kN }
		m { return m }
		N/m { return kN/m }
		Nm { return kNm }
		Nm/m { return kNm/m }
		Pa { return MPa }
	    }
	}
    }
    WarnWin "Error, using an unknown magnitude for the problem units ($system)"
    return ""
}

proc ConvertUnitsToSI { system magnitude } {

    switch $system {
	"N-m" { return 1.0 }
	"N-mm" {
	    switch $magnitude {
		N { return 1.0 }
		m { return 1e-3 }
		N/m { return 1e3 }
		Nm { return 1e-3 }
		Pa { return 1e6 }
	    }
	}
	"Kp-cm" {
	    switch $magnitude {
		N { return 9.8 }
		m { return 1e-2 }
		N/m { return 980.0 }
		Nm { return 0.098 }
		Pa { return 98000.0 }
	    }
	}
	"kN-m-MPa" {
	    switch $magnitude {
		N { return 1.0e3 }
		m { return 1.0 }
		N/m { return 1e3 }
		Nm { return 1e3 }
		Pa { return 1e6 }
	    }
	}
    }
    WarnWin "Error, using an unknown magnitude for the problem units ($system)"
    return ""
}

proc ConvertLengthsUnitsToSI { system } {
    switch $system {
	"m" { return 1.0 }
	"mm" { return 1e-3 }
	"cm" { return 1e-2 }
    }
    WarnWin "Error, using an unknown magnitude for the length units"
    return ""
    
}

proc WarnWin { text { par .gid}} {

    if { $par == "."} {
	set w .__WarnWin
    } elseif { ![winfo exists .gid] } {
	set w .__WarnWin
    } else {
	set w $par.__WarnWin
    }
    tk_dialogRAM  $w Warning $text warning 0 OK
}

proc ChangeToPrefTypeCON { w name } {
    #nothing
}


proc framelabel { w text } {
    frame $w -bd 2 -relief groove
    set i 0
    set parent [winfo parent $w]
    if { $parent == "." } { set parent "" }
    set label $parent.__label$i
    while { [winfo exists $label] } {
	incr i
	set label $parent.__label$i
    }
    label $label -text $text -pady 0
    place $label -in $w -x 10 -y -1 -anchor w -bordermode outside
    after idle raise $label
    return $w
}

proc bindtotal { w event command } {

    bind $w $event $command

    foreach i [winfo children $w] {
	bindtotal $i $event $command
    }
}

proc ChangingElementNumber { base } {

    if { ![winfo exists $base.f1.r1] } { return }
    set elem ConcretePrefs::elem
    set val ""
    catch {
	if { $ConcretePrefs::elem != "" } {
	    set val [.central.s info list_entities Elements $ConcretePrefs::elem]
	}
    }
    if { $val == "" || [lindex $val 0] == "Bad" } {
	$base.f1.r1 conf -state disabled
	$base.f1.r2 conf -state disabled
	return
    }
    set type [lindex $val 4]

    switch $type {
	Linear {
	    $base.f1.r1 conf -state disabled
	    $base.f1.r2 conf -state disabled
	}
	default {
	    $base.f1.r1 conf -state normal
	    $base.f1.r2 conf -state normal
	}
    }
}

proc FillSteelSetsList { w } {
    global ProblemTypePriv

    if { ![winfo exists $w] } { return }

    set state [$w cget -state]
    $w conf -state normal
    $w del 1.0 end
    set indent 0
    set folderclose 0
    foreach text $ConcretePrefs::SteelSetsNames {
	if { [lindex $text 0] == "_end" } {
	    incr indent -1
	    set folderclose 0
	    continue
	}
	if { $folderclose } { continue }
	for { set i 0 } { $i < $indent } { incr i } {
	    $w ins end "   "
	}
	set folder 0
	if { [lindex $text 0] == "_folder" || [lindex $text 0] == "_folderclose"} {
	    incr indent
	    if { [lindex $text 0] == "_folderclose"} { set folderclose 1 }
	    set folder 1

	    if { [lindex $text 0] == "_folderclose"} {
		$w image create end -image $ProblemTypePriv(folder) -padx 2
	    } else {
		$w image create end -image $ProblemTypePriv(openfold) -padx 2
	    }
	    set text [lindex $text 1]
	} else {            
	    $w image create end -image $ProblemTypePriv(steelset) -padx 2
	}
	while { 1 } {
	    set idx [string first D $text]
	    if { $idx == -1 } {
		if { !$folder } {
		    $w ins end $text\n file
		} else { $w ins end $text\n folder }
		break
	    } else {
		if { !$folder } {
		    $w ins end [string range $text 0 [expr $idx-1]] file
		} else { $w ins end [string range $text 0 [expr $idx-1]] folder }
		set text [string range $text [expr $idx+1] end]
		if { !$folder } {
		    $w ins end f "Symbol file"
		} else {
		    $w ins end f "Symbol folder"
		}
	    }
	}
    }
    $w conf -state $state
}

proc ControlSteelSet { w x y } {
    set state [$w cget -state]
    $w conf -state normal
    if { [$w tag nextrange ramsel "@$x,$y linestart" "@$x,$y lineend"] == "" } {
	$w tag add ramsel "@$x,$y linestart" "@$x,$y lineend"
    } else {
	$w tag remove ramsel "@$x,$y linestart" "@$x,$y lineend"
    }
    $w conf -state $state
}

proc ShiftSteelSet { w x y } {
    set state [$w cget -state]
    $w conf -state normal

    set retval [$w tag nextrange ramsel 1.0]
    if { $retval == "" } {
	$w tag add ramsel "@$x,$y linestart" "@$x,$y lineend"
    } else {
	set idx1 [lindex $retval 0]
	set idx2 [lindex $retval 1]

	if { [$w index "@$x,$y linestart"] < $idx1 } { set idx1 [$w index "@$x,$y linestart"] }
	if { [$w index "@$x,$y lineend"] > $idx2 } { set idx2 [$w index "@$x,$y lineend"] }

	while { $idx1 <= $idx2 } {
	    $w tag add ramsel "$idx1 linestart" "$idx1 lineend"
	    set idx1 [expr $idx1+1]
	}
    }
    $w conf -state $state
}

proc BeginMoveSteelSet { w x y } {
    set state [$w cget -state]
    $w conf -state normal
    $w tag remove ramsel 0.0 end
    $w tag add ramsel "@$x,$y linestart" "@$x,$y lineend"
    $w tag add initmove "@$x,$y linestart" "@$x,$y lineend"
    $w conf -state $state
}
    
proc MoveSteelSet { w x y } {
    if { [$w tag nextrange initmove 1.0] == "" } { return }
    set state [$w cget -state]
    $w conf -state normal
    set idx [$w index @$x,$y]
    $w tag remove Underline 1.0 end
    $w tag remove Hilight 1.0 end
    foreach "x1 y1 wi hei" [$w bbox $idx] ""
    
    if { $x >= $x1 && $x <= [expr $x1+$wi] && $y >= $y1 && $y <= [expr $y1+$hei] } { 
	$w conf -cursor double_arrow
	set retval [$w tag nextrange folder "$idx linestart" "$idx lineend"]
	if { $retval != "" && $y < [expr $y1+2*$hei/3] } {
	    $w tag add Hilight "$idx linestart" "$idx lineend"
	} else {
	    $w tag add Underline "$idx linestart" "$idx lineend"
	}
    }
    $w conf -state $state
}


proc SearchYourEnd { list idx } {

    incr idx
    set counter 0
    while { $idx < [llength $list] } {
	set text [lindex $list $idx]
	if { [string match _folder* [lindex $text 0]] } { incr counter }
	if { [lindex $text 0] == "_end" } {
	    if { $counter == 0 } { break }
	    incr counter -1
	}
	incr idx
    }
    return $idx
}

proc EndMoveSteelSet { w x y } {
    if { [$w tag nextrange initmove 1.0] == "" } { return }
    set state [$w cget -state]
    $w conf -state normal
    set idx [$w index @$x,$y]
    foreach "x1 y1 wi hei" [$w bbox $idx] ""
    
    if { $x >= $x1 && $x <= [expr $x1+$wi] && $y >= $y1 && $y <= [expr $y1+$hei] } { 
	set name [ReturnTextFromLabelText $w ramsel.first ramsel.last]
	set idxfrom1 [lsearch $ConcretePrefs::SteelSetsNames $name]
	if { $idxfrom1 == -1 } {
	    set idxfrom1 [lsearch $ConcretePrefs::SteelSetsNames [list _folder $name]]
	    if { $idxfrom1 == -1 } {
		set idxfrom1 [lsearch $ConcretePrefs::SteelSetsNames [list _folderclose $name]]
	    }
	    set idxfrom2 [SearchYourEnd $ConcretePrefs::SteelSetsNames $idxfrom1]
	} else { set idxfrom2 $idxfrom1 }

	set nameto [ReturnTextFromLabelText $w "$idx linestart" "$idx lineend"]
	set idxto [lsearch $ConcretePrefs::SteelSetsNames $nameto]
	if { $idxto == -1 } {
	    set idxto [lsearch $ConcretePrefs::SteelSetsNames [list _folder $nameto]]
	    if { $idxto != -1 && $y >= [expr $y1+2*$hei/3] } {
		if { [lindex [lindex $ConcretePrefs::SteelSetsNames [expr $idxto+1]] 0] == "_end" } {
		    incr idxto
		}
	    }
	    if { $idxto == -1 } {
		set idxto [lsearch $ConcretePrefs::SteelSetsNames [list _folderclose $nameto]]
		if { $y >= [expr $y1+2*$hei/3] } {
		    set idxto [SearchYourEnd $ConcretePrefs::SteelSetsNames $idxto]
		}
	    }
	}

	if { $idxto < $idxfrom1 || $idxto > $idxfrom2 } {
	    set list [lrange $ConcretePrefs::SteelSetsNames $idxfrom1 $idxfrom2]
	    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		    $idxfrom1 $idxfrom2]
	    if { $idxfrom1 < $idxto } { incr idxto [expr -1*($idxfrom2-$idxfrom1+1)] }
	    incr idxto
	    foreach i $list {
		set ConcretePrefs::SteelSetsNames [linsert $ConcretePrefs::SteelSetsNames \
		        $idxto $i]
		incr idxto
	    }
	}
    }
    $w conf -cursor {}
    $w tag remove Underline 1.0 end 
    $w tag remove Hilight 1.0 end 
    $w tag remove initmove 1.0 end 
    $w conf -state $state
}


proc SteelsSetsListProcess { w what { what2 "" } } {


    set name ""
    catch {
	set name [split [ReturnTextFromLabelText $w ramsel.first ramsel.last] "\n"]
    }
    if { [$w index "end-2c linestart"] < 2.0 } {
	$w tag add ramsel 1.0 "1.0 lineend"
	catch {
	    set name [split [ReturnTextFromLabelText $w ramsel.first ramsel.last] "\n"]
	}
    }
    switch $what {
	new { CreateSteelSet $w.new }
	edit {
	    if { $name == "" } {
		WarnWin "Select one steel set to edit"
		return
	    }
	    if { [llength $name] >1 } {
		WarnWin "Select ONLY one steel set to edit"
		return
	    }
	    set name [lindex $name 0]
	    set idx [lsearch $ConcretePrefs::SteelSetsNames [list _folder $name]]
	    if { $idx == -1 } {
		set idx [lsearch $ConcretePrefs::SteelSetsNames [list _folderclose $name]]
	    }
	    if { $idx != -1 } {
		set newname [CreateFolderAsk $w.new $name]
		if { $newname != "" } {
		    set text [lindex $ConcretePrefs::SteelSetsNames $idx]
		    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		            $idx $idx [list [lindex $text 0] $newname]]
		    set idx [SearchYourEnd $ConcretePrefs::SteelSetsNames $idx]
		    set text [lindex $ConcretePrefs::SteelSetsNames $idx]
		    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		            $idx $idx [list [lindex $text 0] $newname]]
		}
		return
	    }
	    CreateSteelSet $w.new $name

	}
	editbutton { 
	    if { $name == "" } {
		return
	    }
	    if { [llength $name] >1 } {
		WarnWin "Select ONLY one steel set to edit"
		return
	    }
	    set name [lindex $name 0]
	    set fol [list _folder $name]
	    set idx [lsearch $ConcretePrefs::SteelSetsNames $fol]
	    if { $idx != -1 } {
		set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		        $idx $idx [list _folderclose $name]]
		return
	    }
	    set fol [list _folderclose $name]
	    set idx [lsearch $ConcretePrefs::SteelSetsNames $fol]
	    if { $idx != -1 } {
		set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		        $idx $idx [list _folder $name]]
		return
	    }
	    CreateSteelSet $w.new $name
	}
	delete {
	    if { $name == "" } {
		WarnWin "Select one steel set to delete"
		return
	    }
	    set sep "\n    "
	    set retval [tk_dialogRAM  $w Warning [concat "Are you sure to delete 'steel sets' "\
		    "or folders?:$sep[join $name $sep]"]\
		    warning 0 OK Cancel]
	    if { $retval == 1 } { return }
	    foreach i $name {
		set idx [lsearch $ConcretePrefs::SteelSetsNames $i]
		if { $idx != -1 } {
		    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		            $idx $idx]
		    unset ConcretePrefs::SteelSets($i)
		    continue
		}
		set idx [lsearch $ConcretePrefs::SteelSetsNames [list _folder $i]]
		if { $idx == -1 } {
		    set idx [lsearch $ConcretePrefs::SteelSetsNames [list _folderclose $i]]
		}
		if { $idx != -1 } {
		    set idxend [SearchYourEnd $ConcretePrefs::SteelSetsNames $idx]
		    for { set j $idx } { $j < $idxend } { incr j } {
		        set text [lindex $ConcretePrefs::SteelSetsNames $j]
		        if { ![string match _* [lindex $text 0]] } {
		            unset ConcretePrefs::SteelSets($text)
		        }
		    }
		    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		            $idx $idxend]
		    continue
		}
	    }
	}
	check {
	    if { $name == "" } {
		WarnWin "Select one steel set to check the section"
		return
	    }
	    if { [llength $name] >1 } {
		WarnWin "Select ONLY one steel set to check the section"
		return
	    }
	    set idx [lsearch $ConcretePrefs::SteelSetsNames [lindex $name 0]]
	    if { $idx == -1 } {
		WarnWin "It is necessary to select one steel set in order to check the section"
		return
	    }
	    ConcretePrefs::Dimension $w $what2 [lindex $name 0]
	}
	folder {
	    set folder [CreateFolderAsk $w.new]
	    if { $folder == "" } { return }
	    lappend ConcretePrefs::SteelSetsNames [list _folder $folder]
	    lappend ConcretePrefs::SteelSetsNames [list _end $folder]
	}

    }
}

proc CreateFrame_Dimension { base } {
    global ProblemTypePriv

    framelabel $base.f1 "select"

    frame $base.f1.f1
    framelabel $base.f1.f2 "shell elems"
    label $base.f1.l -text "Element:"
    entry $base.f1.e -textvar ConcretePrefs::elem -state disabled
    label $base.f1.l2 -text "Point:"
    entry $base.f1.e2 -textvar ConcretePrefs::pointelem -state disabled
    
    button $base.f1.b -text "Sel." -und 2 -command "ConcretePrefs::SelectElement $base.f1"
    button $base.f1.b2 -text "View"  -command "ConcretePrefs::ViewPoint"
    bind [winfo toplevel $base ] <Alt-l> "tkButtonInvoke $base.f1.b"

    GidHelp "$base.f1.l $base.f1.e $base.f1.l2 $base.f1.e2 $base.f1.b" \
	    "Pick button 'Select' and pick a point in the model in the graphical window. "\
	    "The element and strengths "\
	    "associated to that point will be used to dimension the section"

    GidHelp $base.f1.b2 "This command shows in the graphical screen the point that has been "\
	    "previously selected"
    #button $base.f1.b2 -image $ProblemTypePriv(strengths)

#      GidHelp "$base.f1.b2" "Displays the strengths associated with selected element. From there "\
#              "it is possible to transfer the strengths to the user defined strengths"

    radiobutton $base.f1.r1 -text "x' sense" -var ConcretePrefs::xpsense -val 1
    radiobutton $base.f1.r2 -text "y' sense" -var ConcretePrefs::xpsense -val 0

    GidHelp "$base.f1.r1 $base.f1.r2" "if the selected element is a shell element, it is "\
	    "necessary to specify if shell is to be dimensioned in the x' sense or the y' sense"

    trace var ConcretePrefs::elem w "ChangingElementNumber $base ;#"
    ChangingElementNumber $base

    frame $base.f1.f3
    label $base.f1.f3.l1 -text Type:
    tk_optionMenu $base.f1.f3.m ConcretePrefs::elemuse Automatic Support "Beam central" \
	    "Beam extremes" "Beam all" Plate Shell Wall

    GidHelp "$base.f1.f3.l1 $base.f1.f3.m" "This menu permmits to select the typology of the "\
	    "section to be dimensioned.\nOption 'Automatic' tries to decide by itself depending "\
	    "on the strengths. Depending on the option chosen, the changes are:" \
	    "\n\tThe minimum steel that the EHE regulation considers.\n\tIf steel set will "\
	    "be symmetric or no"
    
    $base.f1.f3.m conf -width 12
    label $base.f1.f3.l2 -text Name:
    entry $base.f1.f3.e -textvar ConcretePrefs::elemname

    GidHelp "$base.f1.f3.l2 $base.f1.f3.e" "This is a descriptive name of the section."\
	    " This name will appear in the results report"

    grid $base.f1.f3.l1 -row 1 -column 1 -sticky e
    grid $base.f1.f3.m -row 1 -column 2 -sticky w
    grid $base.f1.f3.l2 -row 1 -column 3 -sticky e
    grid $base.f1.f3.e -row 1 -column 4 -sticky ew
    grid columnconf $base.f1.f3 4 -weight 1


    grid $base.f1.l -in $base.f1.f1 -row 1 -column 1 -sticky e
    grid $base.f1.e -in $base.f1.f1 -row 1 -column 2 -sticky ew
    grid $base.f1.l2 -in $base.f1.f1 -row 2 -column 1 -sticky e
    grid $base.f1.e2 -in $base.f1.f1 -row 2 -column 2 -sticky ew
    grid $base.f1.b -in $base.f1.f1 -row 1 -column 3 -padx 2
    grid $base.f1.b2 -in $base.f1.f1 -row 2 -column 3 -padx 2
#      grid $base.f1.b2 -in $base.f1.f1 -row 1 -column 4 -padx 2
    grid columnconf $base.f1.f1 2 -weight 1

    grid $base.f1.r1 -in $base.f1.f2 -row 1 -column 1
    grid $base.f1.r2 -in $base.f1.f2 -row 1 -column 2

    grid $base.f1.f1  -row 1 -column 1 -pady $GUIState::FramesSeparation -sticky ew \
	    -columnspan 1
    grid $base.f1.f2 -row 1 -column 2 -padx 2 -pady 2 -pady [expr $GUIState::FramesSeparation+2]
    grid $base.f1.f3 -row 2 -column 1 -columnspan 2 -padx 2 -pady 2 -sticky ew
    grid columnconf $base.f1 1 -weight 1

    focus $base.f1.e

    SteelSetsInFrame $base elem

    pack $base.f1 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f3 -side bottom -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f2 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 1 -fill both


}

proc "CreateFrame_User def" { base } {

    framelabel $base.f1 "section"

    frame $base.f1.f1

    label $base.f1.l -text "width:"
    entry $base.f1.e -textvar ConcretePrefs::defaultwidthycm
    label $base.f1.lc -text "cm"
    label $base.f1.l2 -text "  height:"
    entry $base.f1.e2 -textvar ConcretePrefs::defaultwidthzcm
    label $base.f1.lc2 -text "cm"

    GidHelp "$base.f1.l $base.f1.e $base.f1.lc $base.f1.l2 $base.f1.e2 $base.f1.lc2" \
	    "Enter here the width and heigth of the section, expressed in centimeters (cm)"\
	    " in order to describe the user defined section."
    
    frame $base.f1.f3
    label $base.f1.f3.l1 -text Type:
    tk_optionMenu $base.f1.f3.m ConcretePrefs::elemuse Automatic Support "Beam central" \
	    "Beam extremes" "Beam all" Plate Shell Wall
    $base.f1.f3.m conf -width 12
    label $base.f1.f3.l2 -text Name:
    entry $base.f1.f3.e -textvar ConcretePrefs::elemname
    
    GidHelp "$base.f1.f3.l1 $base.f1.f3.m" "This menu permmits to select the typology of the "\
	    "section to be dimensioned.\nOption 'Automatic' tries to decide by itself depending "\
	    "on the strengths. Depending on the option chosen, the changes are:" \
	    "\n\tThe minimum steel that the EHE regulation considers.\n\tIf steel set will "\
	    
    GidHelp "$base.f1.f3.l2 $base.f1.f3.e" "This is a descriptive name of the section."\
	    " This name will appear in the results report"

    grid $base.f1.f3.l1 -row 1 -column 1 -sticky e
    grid $base.f1.f3.m -row 1 -column 2 -sticky w
    grid $base.f1.f3.l2 -row 1 -column 3 -sticky e
    grid $base.f1.f3.e -row 1 -column 4 -sticky ew
    grid columnconf $base.f1.f3 4 -weight 1


    grid $base.f1.l -in $base.f1.f1 -row 1 -column 1 -pady $GUIState::FramesSeparation
    grid $base.f1.e -in $base.f1.f1 -row 1 -column 2 -sticky ew
    grid $base.f1.lc -in $base.f1.f1 -row 1 -column 3 -sticky w
    grid $base.f1.l2 -in $base.f1.f1 -row 1 -column 4
    grid $base.f1.e2 -in $base.f1.f1 -row 1 -column 5 -sticky ew
    grid $base.f1.lc2 -in $base.f1.f1 -row 1 -column 6 -sticky w

    grid columnconf $base.f1.f1 2 -weight 1
    grid columnconf $base.f1.f1 5 -weight 1

    grid $base.f1.f1  -row 1 -column 1 -pady 0 -sticky ew \
	    -columnspan 1
    grid $base.f1.f3 -row 2 -column 1 -columnspan 2 -padx 2 -pady 2 -sticky ew
    grid columnconf $base.f1 1 -weight 1

    focus $base.f1.e

    framelabel $base.f15 "user def. strengths"


    label $base.f15.l1s -text "<0 compression" -font SmallFont
    label $base.f15.l1 -text "Axial force (kN):"
    place $base.f15.l1s -in $base.f15.l1 -anchor n -relx .5 -y -3 -rely 1

    entry $base.f15.e1 -textvar ConcretePrefs::Strengths(axial)
    label $base.f15.l2 -text "M1 (kN\xd7m):"
    entry $base.f15.e2 -textvar ConcretePrefs::Strengths(M1) 
    label $base.f15.l3 -text "M2 (kN\xd7m):"
    entry $base.f15.e3 -textvar ConcretePrefs::Strengths(M2) 
    label $base.f15.l4 -text "Shear (kN):"
    entry $base.f15.e4 -textvar ConcretePrefs::Strengths(shear) 
    label $base.f15.l5 -text "Service factor:"
    entry $base.f15.e5 -textvar ConcretePrefs::Strengths(servicefactor) 
    button $base.f15.b -text Help -command "ConcretePrefs::ViewHelpFor $base strengths.html"

    set sep $GUIState::FramesSeparation
    if { $sep < [expr [font metrics SmallFont -linespace]-3] } {
	set sep [expr [font metrics SmallFont -linespace]-3]
    }

    GidHelp "$base.f15.l1 $base.f15.l1s $base.f15.e1 \
	    $base.f15.l2 $base.f15.e2 \
	    $base.f15.l3 $base.f15.e3 \
	    $base.f15.l4 $base.f15.e4 \
	    $base.f15.l5 $base.f15.e5" \
	    "Press button 'Help' for details.\nNote that Compression is represented by a negative"\
	    "number"

    grid $base.f15.l1 -row 1 -column 1 -sticky e -pady $sep
    grid $base.f15.e1 -row 1 -column 2 -sticky ew
    grid $base.f15.l2 -row 1 -column 3 -sticky e
    grid $base.f15.e2 -row 1 -column 4 -sticky ew -padx 2
    grid $base.f15.l3 -row 2 -column 1 -sticky e
    grid $base.f15.e3 -row 2 -column 2 -sticky ew
    grid $base.f15.l4 -row 2 -column 3 -sticky e
    grid $base.f15.e4 -row 2 -column 4 -sticky ew -padx 2
    grid $base.f15.l5 -row 3 -column 1 -sticky e
    grid $base.f15.e5 -row 3 -column 2 -sticky ew
    grid $base.f15.b -row 3 -column 3 -sticky  w -columnspan 2 -padx 2 -pady 2
    grid columnconf $base.f15 2 -weight 1
    grid columnconf $base.f15 4 -weight 1

    SteelSetsInFrame $base strengths

    pack $base.f1 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f15 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f3 -side bottom -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f2 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 1 -fill both


}

proc SteelSetsInFrame { base what} {

    framelabel $base.f2 "steel sets"

    frame $base.f2.f
    text $base.f2.f.t -xscroll "$base.f2.f.hscr set" -yscroll "$base.f2.f.vscr set" \
	    -wrap none -width 5 -state disabled -cursor ""  -height 3
    $base.f2.f.t tag conf Symbol -font SymbolFont
    $base.f2.f.t tag conf Underline -underline 1
    $base.f2.f.t tag conf Hilight -background steelblue -relief raised -borderwidth 1
    $base.f2.f.t tag conf ramsel -background #72dce4 -relief raised -borderwidth 1

    bind $base.f2.f.t <ButtonPress-1> "BeginMoveSteelSet %W %x %y ; break"
    bind $base.f2.f.t <B1-Motion> "MoveSteelSet %W %x %y ; break"
    bind $base.f2.f.t <ButtonRelease-1> "EndMoveSteelSet %W %x %y ; break"
    bind $base.f2.f.t <B1-Leave> "break"
    bind $base.f2.f.t <Control-ButtonPress-1> "ControlSteelSet %W %x %y ; break"
    bind $base.f2.f.t <Shift-ButtonPress-1> "ShiftSteelSet %W %x %y ; break"

    set steelsetdef [concat "\nOne 'steel set' is one definition of the steel that "\
	    "can be added to one section"\
	    " but that has no information of the concrete section. In this way, one 'steel set'"\
	    " can be adapted to different concrete sections. Given a point in the model, or some"\
	    " user defined strengths, it is possible to check if a given 'steel set' can resist"\
	    " these strengths.\nPick the button and drag the 'steel sets' and folders in order to"\
	    " modify the organization of the database."]

    GidHelp "$base.f2.f.t" "Here are listed all the existing 'steel sets'."\
	    \n$steelsetdef

    scrollbar $base.f2.f.vscr -orient vertical -comm "$base.f2.f.t yview"
    scrollbar $base.f2.f.hscr -orient horizontal -comm "$base.f2.f.t xview"

    trace var ConcretePrefs::SteelSetsNames w "FillSteelSetsList $base.f2.f.t ;#"

    frame $base.f2.f.buts
    button $base.f2.f.buts.b -text New -command "SteelsSetsListProcess $base.f2.f.t new" \
	    -und 0
    button $base.f2.f.buts.b2 -text Edit/Ren -command "SteelsSetsListProcess $base.f2.f.t edit" \
	    -und 0
    button $base.f2.f.buts.b3 -text Delete -command "SteelsSetsListProcess $base.f2.f.t delete" \
	    -und 4
    button $base.f2.f.buts.b4 -text Folder -command "SteelsSetsListProcess $base.f2.f.t folder" \
	    -und 0

    GidHelp "$base.f2.f.buts.b" "Button 'New' permmits to define a new 'steel set'.\n$steelsetdef"
    GidHelp "$base.f2.f.buts.b2" "Button 'Edit/Ren' permmits to modify one 'steel set'"\
	    " or to rename a folder, depending on the item selected.\n$steelsetdef"
    GidHelp "$base.f2.f.buts.b3" "Button 'Delete' permmits to delete one 'steel set' "\
	    "or folder.\n$steelsetdef"
    GidHelp "$base.f2.f.buts.b4" "Button 'Folder' permmits to create a new folder to organize "\
	    "the 'steel sets'.\n$steelsetdef"
    
    set top [winfo toplevel $base]
    bind $top <Alt-n> "tkButtonInvoke $base.f2.f.buts.b"
    bind $base.f2.f.t <Double-1> "SteelsSetsListProcess $base.f2.f.t editbutton; break"
    bind $top <Alt-e> "tkButtonInvoke $base.f2.f.buts.b2"
    bind $top <Alt-t> "tkButtonInvoke $base.f2.f.buts.b3"
    bind $top <Alt-f> "tkButtonInvoke $base.f2.f.buts.b4"

    grid $base.f2.f.buts.b -row 3 -column 1 -pady 2 -padx 2
    grid $base.f2.f.buts.b2 -row 3 -column 2 -pady 2 -padx 2
    grid $base.f2.f.buts.b3 -row 3 -column 3 -pady 2 -padx 2
    grid $base.f2.f.buts.b4 -row 3 -column 4 -pady 2 -padx 2

    grid $base.f2.f.t -row 1 -column 1 -sticky ewns -pady 3
    grid $base.f2.f.vscr -row 1 -column 2 -sticky ns
    grid $base.f2.f.hscr -row 2 -column 1 -sticky ew
    grid $base.f2.f.buts -row 3 -column 1 -columnspan 2
    grid columnconf $base.f2.f 1 -weight 1
    grid rowconf $base.f2.f 1 -weight 1

    grid $base.f2.f -row 1 -column 1 -sticky ewns -pady 3 -pady $GUIState::FramesSeparation
    grid columnconf $base.f2 1 -weight 1
    grid rowconf $base.f2 1 -weight 1

    framelabel $base.f3 "dimension/check"

    button $base.f3.b1 -text Dimension -width 10 -und 0 -command \
	    "ConcretePrefs::Dimension $base $what"
    button $base.f3.b2 -text Check -width 10 -und 4 -command \
	    "SteelsSetsListProcess $base.f2.f.t check $what"
    button $base.f3.b3 -text "View Results" -width 10 -und 0 -command \
	    "ConcretePrefs::ViewLastHtmlFile $base.f2.f.t "

    if { $what == "elem" } {
	set str "the strengths existing in the selected point in the model"
    } else {
	set str "the user defined strengths"
    }

    GidHelp "$base.f3.b1" "This button calculates one 'steel set' that can resist $str."\
	    "It gives the chosen steel, the security of this steel related to the EHE "\
	    "regulations and a complete report of the calculations performed that justify "\
	    "this steel.\nTo view the report, choose 'View results'.\nTo select another"\
	    " 'steel set' for this section, create it with button 'New' and check it with"\
	    " button 'Check'."

    GidHelp "$base.f3.b2" "This button checks if the selected 'steel set' can resist $str."\
	    "It gives the chosen steel, the security of this steel related to the EHE "\
	    "regulations and a complete report of the calculations performed that justify "\
	    "this steel.\nTo view the report, choose 'View results'."

    set projectdir [ConcretePrefs::GiveProjectDir]
    if { $projectdir == "" } { set projectdir MODEL.gid }

    GidHelp "$base.f3.b3" "This button shows the report that justifies the last analysis"\
	    " performed with buttons 'Dimension' or 'Check'.\nTo export this report as "\
	    "HTML create a report with the 'Create' button in the 'Report' tab and use button "\
	    "'Insert' to insert the active results. Then, use button 'Export'.\nUse button "\
	    "'Export DXF' in the 'Report' tab to export the dimensioned section as DXF files."\
	    "\nIt is also possible to obtain the HTML and DXF files by searching in the model "\
	    "directory: $projectdir"


    proc UpdateViewResultsButton { w } {
	if { [winfo exists $w] } {
	    if { $ConcretePrefs::lasthtmlfile == "" } {
		$w conf -state disabled
	    } else {
		$w conf -state normal
	    }
	}
    }

    trace var ConcretePrefs::lasthtmlfile w "UpdateViewResultsButton $base.f3.b3 ;#"
    set ConcretePrefs::lasthtmlfile $ConcretePrefs::lasthtmlfile

    bindtotal $base <Alt-d> "tkButtonInvoke $base.f3.b1 ; break"
    bindtotal $base <Alt-k> "tkButtonInvoke $base.f3.b2 ; break"
    bindtotal $base <Alt-v> "tkButtonInvoke $base.f3.b3 ; break"

    pack $base.f3.b1 $base.f3.b2 $base.f3.b3 -side left -padx 5 -pady $GUIState::FramesSeparation

}

proc CreateFrame_Settings { base } {

    framelabel $base.f2 "other conditions"
    label $base.f2.l1 -text Atmosphere:
    tk_optionMenu $base.f2.m1 ConcretePrefs::atmosphere I IIa IIb IIIa IIIb IIIc IV
    button $base.f2.b -text Help -command "ConcretePrefs::ViewHelpFor $base ambiente.html"
    label $base.f2.l1b -text "Sub atmosphere:"
    tk_optionMenu $base.f2.m1b ConcretePrefs::subatmosphere None Qa Qb Qc H F E
    label $base.f2.l2 -text "Execution control:"
    tk_optionMenu $base.f2.m2 ConcretePrefs::execution factory intense normal reduced
    
    GidHelp "$base.f2.l1 $base.f2.m1 $base.f2.l1b $base.f2.m1b" "Type of atmosphere used in the"\
	    " dimension of the steel. Press button 'Help' for details."

    GidHelp "$base.f2.l2 $base.f2.m2" "Type of execution control used in the dimension"


    grid $base.f2.l1 -row 1 -column 1 -sticky e
    grid $base.f2.m1 -row 1 -column 2 -sticky ew
    grid $base.f2.b -row 1 -column 3 -sticky w
    grid $base.f2.l1b -row 2 -column 1 -sticky e
    grid $base.f2.m1b -row 2 -column 2 -sticky ew
    grid $base.f2.l2 -row 3 -column 1 -sticky e
    grid $base.f2.m2 -row 3 -column 2 -sticky ew
    grid columnconf $base.f2 2 -weight 1

    framelabel $base.f3 "covering & separations"
    frame $base.f3.f

    checkbutton $base.f3.f.cb1 -text "Geom Covering:" -var ConcretePrefs::coveringflag

    proc EvalCoveringFlag { base } {
	if { ![winfo exists $base.f3.f] } { return }
	switch $ConcretePrefs::coveringflag {
	    0 { 
		set ConcretePrefs::covering Automatic
		$base.f3.f.e1 conf -state disabled -fg grey60
	    }
	    1 {
		$base.f3.f.e1 conf -state normal -fg [$base.f3.f.cb1 cget -fg]
		set ConcretePrefs::covering 3.0
	    }
	}
    }

    trace var ConcretePrefs::coveringflag w "EvalCoveringFlag $base ;#"
    entry $base.f3.f.e1 -textvar ConcretePrefs::covering
    label $base.f3.f.l1 -text cm

    GidHelp "$base.f3.f.cb1 $base.f3.f.e1 $base.f3.f.l1" "This is the space left between the outer "\
	    "border of the steel and the exterior edge of the section"


    checkbutton $base.f3.f.cb11 -text "Mec Covering 1:"  -var ConcretePrefs::m1coveringflag

    proc Evalm1CoveringFlag { base } {
	if { ![winfo exists $base.f3.f] } { return }
	switch $ConcretePrefs::m1coveringflag {
	    0 { 
		set ConcretePrefs::m1covering Automatic
		$base.f3.f.e11 conf -state disabled -fg grey60
	    }
	    1 {
		$base.f3.f.e11 conf -state normal -fg [$base.f3.f.cb11 cget -fg]
		set ConcretePrefs::m1covering 4.0
	    }
	}
    }

    trace var ConcretePrefs::m1coveringflag w "Evalm1CoveringFlag $base ;#"
    entry $base.f3.f.e11 -textvar ConcretePrefs::m1covering
    label $base.f3.f.l11 -text cm

    GidHelp "$base.f3.f.cb11 $base.f3.f.e11 $base.f3.f.l11" "This is the distance between the outer "\
	    "border of the steel and the center of gravity of the MAIN steel"


    checkbutton $base.f3.f.cb12 -text "Mec Covering 2:"  -var ConcretePrefs::m2coveringflag

    proc Evalm2CoveringFlag { base } {
	if { ![winfo exists $base.f3.f] } { return }
	switch $ConcretePrefs::m2coveringflag {
	    0 { 
		set ConcretePrefs::m2covering Automatic
		$base.f3.f.e12 conf -state disabled -fg grey60
	    }
	    1 {
		$base.f3.f.e12 conf -state normal -fg [$base.f3.f.cb12 cget -fg]
		set ConcretePrefs::m2covering 4.0
	    }
	}
    }

    trace var ConcretePrefs::m2coveringflag w "Evalm2CoveringFlag $base ;#"
    entry $base.f3.f.e12 -textvar ConcretePrefs::m2covering
    label $base.f3.f.l12 -text cm

    GidHelp "$base.f3.f.cb12 $base.f3.f.e12 $base.f3.f.l12" "This is the distance between the outer "\
	    "border of the steel and the center of gravity of the SECONDARY steel"

    checkbutton $base.f3.f.cb2 -text Separations: -var ConcretePrefs::separationsflag

    proc EvalSeparationsFlag { base } {
	if { ![winfo exists $base.f3.f] } { return }
	switch $ConcretePrefs::separationsflag {
	    0 { 
		set ConcretePrefs::separations Automatic
		$base.f3.f.e2 conf -state disabled -fg grey60
	    }
	    1 {
		$base.f3.f.e2 conf -state normal -fg [$base.f3.f.cb2 cget -fg]
		set ConcretePrefs::separations 2.0
	    }
	}
    }

    trace var ConcretePrefs::separationsflag w "EvalSeparationsFlag $base ;#"
    entry $base.f3.f.e2 -textvar ConcretePrefs::separations
    label $base.f3.f.l2 -text cm

    GidHelp "$base.f3.f.cb2 $base.f3.f.e2 $base.f3.f.l2" "This is the minimum space left between "\
	    "steels when dimensioning. This space can be increased for big steels"

    grid $base.f3.f.cb1 -row 1 -column 1 -sticky w
    grid $base.f3.f.e1 -row 1 -column 2 -sticky ew
    grid $base.f3.f.l1 -row 1 -column 3
    grid $base.f3.f.cb11 -row 2 -column 1 -sticky w
    grid $base.f3.f.e11 -row 2 -column 2 -sticky ew
    grid $base.f3.f.l11 -row 2 -column 3
    grid $base.f3.f.cb12 -row 3 -column 1 -sticky w
    grid $base.f3.f.e12 -row 3 -column 2 -sticky ew
    grid $base.f3.f.l12 -row 3 -column 3
    grid $base.f3.f.cb2 -row 4 -column 1 -sticky w
    grid $base.f3.f.e2 -row 4 -column 2 -sticky ew
    grid $base.f3.f.l2 -row 4 -column 3
    grid columnconf $base.f3.f 2 -weight 1

    grid $base.f3.f -sticky nsew -column 1 -row 1 -pady $GUIState::FramesSeparation 
    grid columnconf $base.f3 1 -weight 1
    grid rowconf $base.f3 1 -weight 1

    framelabel $base.f4 "Shrinkage"
    label $base.f4.l -text "Alpha buckling:"
    entry $base.f4.e -textvar ConcretePrefs::alpha

    checkbutton $base.f4.cb -text "Beam length:" -var ConcretePrefs::BeamLengthFlag

    proc EvalBeamLengthFlag { base } {
	if { ![winfo exists $base.f4.cb] } { return }
	switch $ConcretePrefs::BeamLengthFlag {
	    0 { 
		set ConcretePrefs::BeamLength Automatic
		$base.f4.e2 conf -state disabled -fg grey60
	    }
	    1 {
		$base.f4.e2 conf -state normal -fg [$base.f4.cb cget -fg]
		set ConcretePrefs::BeamLength 1.0
	    }
	}
    }

    trace var ConcretePrefs::BeamLengthFlag w "EvalBeamLengthFlag $base ;#"
    entry $base.f4.e2 -textvar ConcretePrefs::BeamLength
    label $base.f4.l2 -text m

    GidHelp "$base.f4.cb $base.f4.e2 $base.f4.l2" "This is the length of the beam or shell"\
	    " used for the Shrinkage analysis. It is only necessary to enter when the length"\
	    " of the mesh element is not the same than the length of the structural element"

    grid $base.f4.l -row 1 -column 1 -pady 3  -padx 2
    grid $base.f4.e -row 1 -column 2 -sticky ew -padx 2
    grid $base.f4.cb -row 2 -column 1 -sticky w
    grid $base.f4.e2 -row 2 -column 2 -sticky ew
    grid $base.f4.l2 -row 2 -column 3 -sticky w
    grid columnconf $base.f4 2 -weight 1

    GidHelp "$base.f4.l $base.f4.e" "This is the relationship between the real length of "\
	    "the support and the length used for shrinkage (pandeo) analysis"

    framelabel $base.f5 "output options"
    checkbutton $base.f5.l -text "Symbols as images in HTML" -var ConcretePrefs::SymbolsAsImages
    #checkbutton $base.f5.l2 -text "Output a MS Word file" -var ConcretePrefs::OutputRTF
    grid $base.f5.l -row 1 -column 1 -pady 3 -sticky w -padx 2
    #grid $base.f5.l2 -row 2 -column 1 -pady 3 -sticky w -padx 2
    grid columnconf $base.f5 1 -weight 1

    GidHelp $base.f5.l "Some browsers, like Netscape, do not support the special font 'Symbol'"\
	" to display greek letters and special symbols. In these cases, greek letters "\
	"can be output as images"
    
    GidHelp $base.f5.l2 "If this variable is set, reports are output in MS Word format. If not,"\
	" it is putput as HTML"

#      pack $base.f1 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f2 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f3 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f4 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f5 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x

}

proc CreateFrame_Materials { base } {

    framelabel $base.f1 "materials"
    label $base.f1.l1 -text "Steel:"
    tk_optionMenu $base.f1.m1 ConcretePrefs::steeltype "B400-S" "B500-S"
    
    label $base.f1.l2 -text "Concrete (default):"
    tk_optionMenu $base.f1.m2 ConcretePrefs::concretetype HA-20 HA-25 HA-30 HA-35 HA-40 HA-45 HA-50

    GidHelp "$base.f1.l1 $base.f1.m1" "This is the type of steel used in the dimension. The"\
	    " number indicates the characteristic resistance of the steel in N/mm2"

    GidHelp "$base.f1.l2 $base.f1.m2" "This is the type of concrete that will be used in the "\
	    "dimension, for all the sections that do NOT have already concrete properties. The"\
	    " number indicates the characteristic resistance of the concrete in N/mm2"

    grid $base.f1.l1 -row 1 -column 1 -sticky e
    grid $base.f1.m1 -row 1 -column 2 -sticky ew
    grid $base.f1.l2 -row 2 -column 1 -sticky e
    grid $base.f1.m2 -row 2 -column 2 -sticky ew
    grid columnconf $base.f1 2 -weight 1


    framelabel $base.f2 "steel long."
    set longsteels "6 8 10 12 14 16 20 25 32 40"

    set icount 1
    foreach i $longsteels {
	checkbutton $base.f2.b$i -text $i -var ConcretePrefs::steellong$i
	if { $icount <= 5 } {
	    grid $base.f2.b$i -row 1 -column $icount -sticky ew
	} else {
	    grid $base.f2.b$i -row 2 -column [expr $icount-5] -sticky ew
	}
	GidHelp $base.f2.b$i "The activated steels will be the only ones used as longitudinal "\
		"steel when dimensioning a section. Also, they are the ones that can be selected "\
		"when creating a new steel set"
	incr icount
    }

    framelabel $base.f3 "steel transv."
    set longsteels "6 8 10 12"

    set icount 1
    foreach i $longsteels {
	checkbutton $base.f3.b$i -text $i -var ConcretePrefs::steeltransv$i
	if { $icount <= 5 } {
	    grid $base.f3.b$i -row 1 -column $icount -sticky ew
	} else {
	    grid $base.f3.b$i -row 2 -column [expr $icount-5] -sticky ew
	}
	GidHelp $base.f3.b$i "The activated steels will be the only ones used as "\
		"transversal steel when "\
		"dimensioning a section. Also, they are the ones that can be selected "\
		"when creating a new steel set"
	incr icount
    }

    pack $base.f1 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f2 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f3 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x

}

proc WarnWinReport { text w } {

    set ww $w.__WarnWin
    set retval [tk_dialogRAM  $ww Warning $text warning 0 OK "View report"]
    if { $retval == 1 } {
	ConcretePrefs::ViewReportHtmlFile $w
    }
}

proc CreateFrame_Report { base } {

    
    framelabel $base.f1 "global report"

    label $base.f1.l1 -text Title:
    entry $base.f1.e1 -textvar ConcretePrefs::reporttitle

    GidHelp "$base.f1.l1 $base.f1.e1" "This is the title of the report that will be created "\
	    " with button 'Create'."

    frame $base.f1.buts0
    button $base.f1.buts0.b1 -text Description... -width 15 -command "EnterDescription $base._desc"
    pack $base.f1.buts0.b1 -side left 

    GidHelp "$base.f1.buts0.b1" "Here can be entered the description of the analysis made. It"\
	    " will appear in the report"

    frame $base.f1.buts
    button $base.f1.buts.b1 -text Create -width 7 -command \
	    "ConcretePrefs::CreateReport $base.f1"
    button $base.f1.buts.b2 -text View -width 7 -command \
	    "ConcretePrefs::ViewReportHtmlFile $base.f1"
    menubutton $base.f1.buts.b3 -text Export -width 7 -menu \
	$base.f1.buts.b3.m -relief [$base.f1.buts.b2 cget -relief] \
	-bd [$base.f1.buts.b2 cget -bd]

    menu $base.f1.buts.b3.m
    $base.f1.buts.b3.m add command -label "As HTML" -command \
	"ConcretePrefs::ExportReport $base.f1 HTML"
    $base.f1.buts.b3.m add command -label "As Word" -command \
	"ConcretePrefs::ExportReport $base.f1 Word"

    GidHelp "$base.f1.buts.b1" "This button will create a new report of the analysis."\
	    "It can be filled afterwards by ussing buttons 'Insert'."

    GidHelp "$base.f1.buts.b2" "This button launches a viewre that will display the created report"\
	    "\nIt must be previusly created with button 'Create'."

    set projectdir [ConcretePrefs::GiveProjectDir]
    if { $projectdir == "" } { set projectdir MODEL.gid }

    GidHelp "$base.f1.buts.b3" "This button exports the report as an HTML or Word file."\
	    "\nIt is also possible to obtain the HTML and DXF files by searching in the model "\
	    "directory: $projectdir"
    

    proc UpdateViewReportButton { args } {
	foreach w $args {
	    if { [winfo exists $w] } {
		if { $ConcretePrefs::reportfilename == "" } {
		    $w conf -state disabled
		} else {
		    $w conf -state normal
		}
	    }
	}
    }

    trace var ConcretePrefs::reportfilename w "UpdateViewReportButton $base.f1.buts.b2 \
	    $base.f1.buts.b3 ;#"
    set ConcretePrefs::reportfilename $ConcretePrefs::reportfilename

    grid $base.f1.buts.b1 -row 1 -column 1 -padx 2 -pady 2
    grid $base.f1.buts.b2 -row 1 -column 2 -padx 2 -pady 2
    grid $base.f1.buts.b3 -row 1 -column 3 -padx 2 -pady 2

    grid $base.f1.l1 -row 1 -column 1 -sticky e -pady $GUIState::FramesSeparation
    grid $base.f1.e1 -row 1 -column 2 -sticky ew -padx 2
    grid $base.f1.buts0 -row 2 -column 1 -columnspan 2 -sticky w -padx 2
    grid $base.f1.buts -row 3 -column 1 -columnspan 2 -sticky ew
    grid columnconf $base.f1 2 -weight 1

    framelabel $base.f2 "insert current view"
    label $base.f2.l1 -text Caption:
    entry $base.f2.e1 -textvar ConcretePrefs::reportlastimgcaption

    frame $base.f2.buts
    button $base.f2.buts.b1 -text Insert -width 8 -command \
	    "ConcretePrefs::AddCurrentViewToReport $base"
    button $base.f2.buts.b2 -text "Delete last" -width 8 -command \
	    "ConcretePrefs::DeleteLastFromReport $base image"

    GidHelp "$base.f2.l1 $base.f2.e1" "When inserting an image in the report with button 'Insert',"\
	    " caption is the text that appears just below the image, describing it."

    GidHelp "$base.f2.buts.b1" "This option inserts current view of the model at the end"\
	    " of the report."
    GidHelp "$base.f2.buts.b2" "This button deletes from the report the last inserted image"

    grid $base.f2.buts.b1 -row 1 -column 1 -padx 2 -pady 2
    grid $base.f2.buts.b2 -row 1 -column 2 -padx 2 -pady 2
    
    grid $base.f2.l1 -row 1 -column 1 -sticky e -pady $GUIState::FramesSeparation
    grid $base.f2.e1 -row 1 -column 2 -sticky ew -padx 2
    grid $base.f2.buts -row 3 -column 1 -columnspan 2 -sticky ew
    grid columnconf $base.f2 2 -weight 1


    framelabel $base.f3 "insert current results"

    frame $base.f3.buts
    button $base.f3.buts.b1 -text Insert -width 8 -command \
	    "ConcretePrefs::AddCurrentResultsToReport $base"
    button $base.f3.buts.b2 -text "Delete last" -width 8 -command \
	    "ConcretePrefs::DeleteLastFromReport $base results"

    GidHelp "$base.f3.buts.b1" "This option inserts last results obtained with buttons"\
	    " 'Dimension' or 'Check' at the end of the report."
    GidHelp "$base.f3.buts.b2" "This button deletes from the report the last inserted results"


    grid $base.f3.buts.b1 -row 1 -column 1 -padx 2 -pady 2
    grid $base.f3.buts.b2 -row 1 -column 2 -padx 2 -pady 2
    
    grid $base.f3.buts -row 1 -column 1 -columnspan 2 -sticky ew -pady $GUIState::FramesSeparation
    grid columnconf $base.f3 2 -weight 1

    framelabel $base.f4 "export dxf"

    frame $base.f4.buts
    button $base.f4.buts.b1 -text "Export DXF" -width 12 -command \
	    "ConcretePrefs::ExportDXF $base"

    GidHelp "$base.f4.buts.b1" "This button exports as a DXF file the drawing of the last"\
	    " sections obtained with buttons 'Dimension' or 'Check'"

    grid $base.f4.buts.b1 -row 1 -column 1 -padx 2 -pady 2
    
    grid $base.f4.buts -row 1 -column 1 -columnspan 2 -sticky ew -pady $GUIState::FramesSeparation
    grid columnconf $base.f4 2 -weight 1

    pack $base.f1 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f2 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f3 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x
    pack $base.f4 -side top -padx 5 -pady $GUIState::FramesSeparation -expand 0 -fill x

} 

proc AskForWidth { w } {
    global tkPriv tcl_platform

    toplevel $w
    wm title $w "Dimensions"

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    framelabel $w.f "section dimensions"
    label $w.f.l1 -text "Width Y:"
    entry $w.f.e1 -textvar ConcretePrefs::defaultwidthycm
    label $w.f.l1b -text "cm"

    label $w.f.l2 -text "Width Z:"
    entry $w.f.e2 -textvar ConcretePrefs::defaultwidthzcm
    label $w.f.l2b -text "cm"

    grid $w.f.l1 -row 1 -column 1  -pady $GUIState::FramesSeparation -padx 2
    grid $w.f.e1 -row 1 -column 2 -sticky ew
    grid $w.f.l1b -row 1 -column 3 -sticky w
    grid $w.f.l2 -row 2 -column 1
    grid $w.f.e2 -row 2 -column 2 -sticky ew
    grid $w.f.l2b -row 2 -column 3 -sticky w
    grid columnconf $w.f 2 -weight 1

    button $w.ok -text OK -width 5 -command "set tkPriv(afw_button) ok" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(afw_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w.f.e1 <Return> "tkButtonInvoke $w.ok"
    bind $w.f.e2 <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.cancel"

    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation
    grid $w.ok -row 2 -column 1 
    grid $w.cancel -row 2 -column 2
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    tkwait variable tkPriv(afw_button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }
    return $tkPriv(afw_button)
}

proc SavePrefs { w } {
    global tkPriv tcl_platform

    toplevel $w
    wm title $w "Save preferences"

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    framelabel $w.f "save preferences on..."

    radiobutton $w.f.r1 -text "Rambshell" -var ConcretePrefs::saveprefsonproblemtype -value 1
    radiobutton $w.f.r2 -text "local project" -var ConcretePrefs::saveprefsonproblemtype -value 0
    
    label $w.f.l -text [concat "Note: If 'Rambshell' is chosen, new projects will have access to "\
	    "these preferences."] -justify left -wraplength 250 -height 3
    bind $w.f.l <Configure> "$w.f.l conf -wraplength \[winfo width $w.f.l]"

    grid $w.f.r1 -row 1 -column 1  -pady $GUIState::FramesSeparation -padx 2 -sticky w
    grid $w.f.r2 -row 2 -column 1 -sticky w
    grid $w.f.l  -row 3 -column 1 -sticky ew
    grid columnconf $w.f 1 -weight 1

    button $w.ok -text OK -width 5 -command "set tkPriv(sp_button) Save" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(sp_button) Cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.cancel"

    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $w.cancel -row 2 -column 2 -pady 2 -sticky w
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    tkwait variable tkPriv(sp_button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    if { $tkPriv(sp_button) == "Save" } {
	ConcretePrefs::Save
    }
}

proc SteelSetsOkPrefs { w } {
    global tkPriv tcl_platform

    toplevel $w
    wm title $w "Accept new steel sets"

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.f -bd 2 -relief groove

    label $w.f.l -text [concat "You are editing one steel set and you have changed its name. What"\
	    " do you want to do?"] -width 50 -justify left -height 2
    bind $w.f.l <Configure> "$w.f.l conf -wraplength \[winfo width $w.f.l]"

    checkbutton $w.f.r1 -text "Maintain old set" -var ConcretePrefs::steelsetsmaintainoldset
    checkbutton $w.f.r2 -text "Create new set" -var ConcretePrefs::steelsetscreatenewset

    grid $w.f.l  -row 1 -column 1 -sticky ew -pady $GUIState::FramesSeparation
    grid $w.f.r1 -row 2 -column 1  -pady $GUIState::FramesSeparation -padx 2 -sticky w
    grid $w.f.r2 -row 3 -column 1 -sticky w -padx 2
    grid columnconf $w.f 1 -weight 1

    button $w.ok -text OK -width 5 -command "set tkPriv(ssop_button) ok" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(ssop_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.cancel"

    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $w.cancel -row 2 -column 2 -pady 2 -sticky w
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    tkwait variable tkPriv(ssop_button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    return $tkPriv(ssop_button)
}

proc CreateFolderAsk { w  { oldname "" } } {
    global tkPriv tcl_platform ProblemTypePriv

    toplevel $w
    if { $oldname == "" } {
	wm title $w "Create steel sets folder"
    } else {
	wm title $w "Rename steel sets folder"
    }

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.f -bd 2 -relief groove

    label $w.f.l -text [concat "Enter new folder name:"] -width 50 -justify left -anchor w

    createtextaslabel $w.f.e
    $w.f.e conf -width 30

    if { $oldname != "" } {
	PutTextinLabelText $w.f.e $oldname
	$w.f.e tag add sel 1.0 "1.0 lineend"
    }

    button $w.f.b -image $ProblemTypePriv(phi) -command "\
	    $w.f.e insert insert f Symbol"

    label $w.f.l2 -text [concat "Note: You can later move the folders and steel sets positions" \
	    " by dragging them with the mouse"] -width 50 -justify left -height 2
    bind $w.f.l2 <Configure> "$w.f.l2 conf -wraplength \[winfo width $w.f.l2]"
   
    grid $w.f.l  -row 1 -column 1 -sticky w -pady $GUIState::FramesSeparation -padx 5 -columnspan 2
    grid $w.f.e -row 2 -column 1  -pady $GUIState::FramesSeparation -padx 2 -sticky e
    grid $w.f.b -row 2 -column 2  -pady $GUIState::FramesSeparation -padx 0 -sticky w
    grid $w.f.l2 -row 3 -column 1 -sticky w -padx 2 -sticky ew -columnspan 2


    button $w.ok -text OK -width 5 -command "set tkPriv(cf_button) ok" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(cf_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w.f.e <Return> "tkButtonInvoke $w.ok ; break"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w.f.e <Alt-o> "tkButtonInvoke $w.ok"
    bind $w.f.e <Alt-c> "tkButtonInvoke $w.cancel"

    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $w.cancel -row 2 -column 2 -pady 2 -sticky w
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.f.e
    after 100 catch [list "focus -force $w.f.e"]

    while 1 {
	tkwait variable tkPriv(cf_button)

	set retval ""
	if { $tkPriv(cf_button) == "ok" } {
	    set retval [ReturnTextFromLabelText $w.f.e 1.0 end]
	    if { $retval == "" } {
		WarnWin "error: it is necessary to enter a folder name"
	    } elseif { [string match _* $retval] } {
		WarnWin "error: folder name cannot begin with '_'"
	    } else { break }
	} else { break }
    }
    
    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    return $retval
}

proc PutHtmlInText { w text { clear yes } } {

    set state [$w cget -state]
    $w conf -state normal
    if { $clear == "yes" } {
	$w del 1.0 end
    }

    regsub -all </p><p> $text \n text
    regsub -all <p> $text {} text
    regsub -all </p> $text {} text
    while { 1 } {
	if { ![regexp -indices {<([^>]*)>} $text tag tagname] } {
	    $w ins end $text
	    break
	}
	$w ins end [string range $text 0 [expr [lindex $tag 0]-1]]
	set tagname [string range $text [lindex $tagname 0] [lindex $tagname 1]]
	regexp -indices {</[^>]*>} $text endtag
	$w ins end [string range $text [expr [lindex $tag 1]+1] [expr [lindex $endtag 0]-1]] $tagname
	set text [string range $text [expr [lindex $endtag 1]+1] end]
    }
    $w conf -state $state

}

proc ReturnHtmlFromText { w } {

    set text ""
    foreach "key val idx" [$w dump -tag -text 1.0 end] {
	switch $key {
	    text { 
		if { $val != "\n" } {
		    append text $val 
		}
	    }
	    tagon { append text <$val> }
	    tagoff { append text </$val> }
	}
    }
    set text "<p>$text</p>"
    regsub -all \n $text </p><p> text
    return $text
}

proc EnterDescription { w } {
    global tkPriv tcl_platform ProblemTypePriv

    toplevel $w
    wm title $w "Report description"

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.f -bd 2 -relief groove


    frame $w.f.buts
    
    button $w.f.buts.b -image $ProblemTypePriv(phi) -command "\
	    $w.f.t insert insert f sym"
    pack $w.f.buts.b -side left


    text $w.f.t -xscroll "$w.f.hscr set" -yscroll "$w.f.vscr set" \
	    -width 60 
    $w.f.t tag conf sym -font SymbolFont

    PutHtmlInText $w.f.t $ConcretePrefs::reportdescription yes

    bind $w.f.t <KeyPress> {
	if { [regexp {[a-zA-Z]} %A] } {
	    catch {
		if {[%W compare sel.first <= insert]
		&& [%W compare sel.last >= insert]} {
		    %W delete sel.first sel.last
		}
	    }
	    %W insert insert %A ""
	    break 
	}
    }

    scrollbar $w.f.vscr -orient vertical -comm "$w.f.t yview"
    scrollbar $w.f.hscr -orient horizontal -comm "$w.f.t xview"

    grid $w.f.buts  -row 1 -column 1 -sticky w -pady $GUIState::FramesSeparation -padx 5 -columnspan 2
    grid $w.f.t -row 2 -column 1  -padx 2 -sticky nsew
    grid $w.f.vscr -row 2 -column 2 -sticky ns
    grid $w.f.hscr -row 3 -column 1 -sticky ew -padx 2 
    grid columnconf $w.f 1 -weight 1
    grid rowconf $w.f 2 -weight 1

    frame $w.dbuts
    button $w.ok -text OK -width 5 -command "set tkPriv(ed_button) ok" -und 0
    button $w.clear -text Clear -width 5 -command "$w.f.t del 1.0 end" -und 1
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(ed_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w.f.t <Alt-o> "tkButtonInvoke $w.ok"
    bind $w.f.t <Alt-l> "tkButtonInvoke $w.clear"
    bind $w.f.t <Alt-c> "tkButtonInvoke $w.cancel"

    grid $w.f -row 1 -column 1  -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.dbuts -row 2 -column 1
    grid $w.ok -in $w.dbuts -row 2 -column 1 -sticky e -pady 3 -padx 2
    grid $w.clear -in $w.dbuts -row 2 -column 2 -pady 2 -sticky w -padx 2
    grid $w.cancel -in $w.dbuts -row 2 -column 3 -pady 2 -sticky w -padx 2

    grid columnconf $w 1 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.f.t
    after 100 catch [list "focus -force $w.f.e"]

    while 1 {
	tkwait variable tkPriv(ed_button)

	set retval ""
	if { $tkPriv(ed_button) == "ok" } {
	    set ConcretePrefs::reportdescription [ReturnHtmlFromText $w.f.t]
	    #WarnWinText ---$ConcretePrefs::reportdescription---
	    break
	} else { break }
    }
    
    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    return $retval
}

proc DisplayStrengths { w elmnum { what create } } {
    global tkPriv tcl_platform ProblemTypePriv

    if { $what == "update" && ![winfo exists $w] } { return }

    if { $what == "create" } {
	catch { destroy $w }

	if { [info procs InitWindow] != "" } {
	    InitWindow $w "Element strengths" PostElemStrengthsWindowGeom
	} else {
	    toplevel $w
	    wm title $w "Element strengths"
	}
	frame $w.f -bd 2 -relief groove

	text $w.f.t -xscroll "$w.f.hscr set" -yscroll "$w.f.vscr set" \
		-width 60 
	$w.f.t tag conf sym -font SymbolFont

	scrollbar $w.f.vscr -orient vertical -comm "$w.f.t yview"
	scrollbar $w.f.hscr -orient horizontal -comm "$w.f.t xview"

	grid $w.f.t -row 1 -column 1  -padx 2 -sticky nsew -pady $GUIState::FramesSeparation
	grid $w.f.vscr -row 1 -column 2 -sticky ns
	grid $w.f.hscr -row 2 -column 1 -sticky ew -padx 2 
	grid columnconf $w.f 1 -weight 1
	grid rowconf $w.f 1 -weight 1

	frame $w.dbuts
	button $w.transfer -text Transfer -width 5 -command "set tkPriv(ed_button) ok" -und 0
	button $w.close -text Close -width 5 -command "$w.f.t del 1.0 end" -und 1

	bind $w.ok <Return> "tkButtonInvoke $w.close"
	bind $w <Escape> "tkButtonInvoke $w.close"
	bind $w.f.t <Alt-t> "tkButtonInvoke $w.transfer"
	bind $w.f.t <Alt-c> "tkButtonInvoke $w.close"

	grid $w.f -row 1 -column 1  -pady $GUIState::FramesSeparation -sticky ewns \
		-padx 2
	grid $w.dbuts -row 2 -column 1
	grid $w.transfer -in $w.dbuts -row 2 -column 1 -sticky e -pady 3 -padx 2
	grid $w.close -in $w.dbuts -row 2 -column 2 -pady 2 -sticky w -padx 2

	grid columnconf $w 1 -weight 1
	grid rowconf $w 1 -weight 1
	
	wm withdraw $w
	update idletasks
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
	focus $w.close
    }
}

proc SteelSetsOkDiffNameValue { w name value } {
    global tkPriv tcl_platform

    toplevel $w
    wm title $w "Accept new steel sets"

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.f -bd 2 -relief groove

    label $w.f.l -text "You are going to accept one steel set with the following data. Proceed?"\
	    -width 50 -justify left -height 2
    bind $w.f.l <Configure> "$w.f.l conf -wraplength \[winfo width $w.f.l]"

    label $w.f.lname -text Name:
    createtextaslabel $w.f.e1
    $w.f.e1 conf -state disabled -bd 0
    PutTextinLabelText $w.f.e1 $name

    grid $w.f.l  -row 1 -column 1 -sticky ew -pady $GUIState::FramesSeparation -columnspan 2
    grid $w.f.lname -row 2 -column 1  -pady $GUIState::FramesSeparation -padx 2 -sticky e
    grid $w.f.e1 -row 2 -column 2  -padx 2 -sticky ew

    if { $name != $value } {
	label $w.f.lval -text "Steel set:"
	createtextaslabel $w.f.e2
	$w.f.e2 conf -state disabled -bd 0
	PutTextinLabelText $w.f.e2 $value
	grid $w.f.lval -row 3 -column 1  -padx 2 -sticky e
	grid $w.f.e2 -row 3 -column 2  -padx 2 -sticky ew -pady 2
    }

    grid columnconf $w.f 2 -weight 1

    button $w.ok -text OK -width 5 -command "set tkPriv(ssod_button) ok" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(ssod_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.cancel"
    
    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $w.cancel -row 2 -column 2 -pady 2 -sticky w
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    tkwait variable tkPriv(ssod_button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    return $tkPriv(ssod_button)
}

proc ResultDimension { w base file what } {
    global tkPriv tcl_platform ProblemTypePriv

    if { [catch {
	set fin [open $file r]
    }] } {
	WarnWin "The process has had one error"
	return
    }
    
    toplevel $w
    if { $what == "dimension" } {
	wm title $w "Dimension results"
    } else {
	wm title $w "Check results"
    }
    
    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }
    
    frame $w.f -bd 2 -relief groove
    
    if { $what == "dimension" } {
	label $w.f.l -text "The result of the dimension of the section is:" -width 50 -justify left \
		-anchor w
    } else {
	label $w.f.l -text "The result of the checking of the section is:" -width 50 -justify left \
		-anchor w
    }
    bind $w.f.l <Configure> "$w.f.l conf -wraplength \[winfo width $w.f.l]"

    frame $w.f.t

    
    label $w.f.l1 -text "Result type" -bd 2 -relief ridge -font BoldFont
    label $w.f.v1 -text "Additional security factor" -bd 2 -relief ridge -font BoldFont
    grid $w.f.l1 -row 1 -column 1 -sticky ew -in $w.f.t
    grid $w.f.v1 -row 1 -column 2 -sticky ew -in $w.f.t

    set message ""

    set sections ""
    set i 2
    while { ![eof $fin] } {
	gets $fin aa
	if { [regexp {NumSections=([0-9]+)} $aa {} NumSections] } { continue }
	if { [regexp {SEC[ ]*([0-9]+):[ ]*CS[ ]*([^ ]+)[ ]*(.+)} $aa {} secnum name value] } {
	    switch $name {
		MinLongitudinal { set text "Minimum longitudinal steel" }
		Longitudinal { set text "Longitudinal steel" }
		Fisuration { set text "Fisuration" }
	    }
	    label $w.f.l$i -text $text -bd 2 -relief ridge
	    label $w.f.v$i -text $value -bd 2 -relief ridge
	    if { ![string is double -strict $value] || $value < 1.0 } {
		$w.f.l$i conf -fg red ; $w.f.v$i conf -fg red
	    }
	    grid $w.f.l$i -row $i -column 1 -sticky ew -in $w.f.t
	    grid $w.f.v$i -row $i -column 2 -sticky ew -in $w.f.t
	    incr i
	} elseif { [regexp {CS[ ]*([^ ]+)[ ]*(.+)} $aa {} name value] } {
	    switch $name {
		Transversal { set text "Transversal steel" }
		Global { set text "Global security factor" }
	    }
	    label $w.f.l$i -text $text -bd 2 -relief ridge
	    label $w.f.v$i -text $value -bd 2 -relief ridge
	    if { ![string is double -strict $value] || $value < 1.0 } {
		$w.f.l$i conf -fg red ; $w.f.v$i conf -fg red
	    }
	    if { $name == "Global" } {
		$w.f.l$i conf -font BoldFont -bd 3; $w.f.v$i conf -font BoldFont -bd 3
	    }
	    grid $w.f.l$i -row $i -column 1 -sticky ew -in $w.f.t
	    grid $w.f.v$i -row $i -column 2 -sticky ew -in $w.f.t
	    incr i
	} elseif { [regexp {Message[ ]+(.*)} $aa {} message] } {
	    set message [string trim $message]
	} elseif { [regexp {Section[0-9]*[ ]+(.*)} $aa {} sec] } {
	    lappend sections [string trim $sec]
	}
    }
    label $w.f.l$i -text "NOTE: A coefficient equal or bigger than 1.0 means security in the results"\
	    -bd 2 -relief ridge -width 50 -justify left \
	    -anchor w -height 2
    bind $w.f.l$i <Configure> "$w.f.l$i conf -wraplength \[winfo width $w.f.l$i]"

    grid $w.f.l$i -row $i -column 1 -sticky e -in $w.f.t -columnspan 2

    close $fin

    if { $message != "" } {
	incr i
	label $w.f.l$i -text "$message" -width 50 -justify left \
		-anchor w -height 2 -fg red
	bind $w.f.l$i <Configure> "$w.f.l$i conf -wraplength \[winfo width $w.f.l$i]"
	grid $w.f.l$i -row $i -column 1 -sticky e -in $w.f.t -columnspan 2 -pady 5
    }

    proc EnterNewSteelsSetsFromResults { w entry value} {
	set name [ReturnTextFromLabelText $entry 1.0 end]
	set retval [SteelSetsOkDiffNameValue $w $name $value]
	if { $retval == "ok" } {
	    set retval 0
	    set isnew 1
	    if { [info exists ConcretePrefs::SteelSets($name)] } {
		set retval [tk_dialogRAM  $w Warning \
		        "Steel set '$name' already exists. Overwrite?"\
		        warning 0 OK Cancel]
		set isnew 0
	    }
	    if { $retval == 0 } {
		if { $isnew } { lappend ConcretePrefs::SteelSetsNames $name }
		set ConcretePrefs::SteelSets($name) $value
	    }
	}
    }

    if { $sections != "" } {
	incr i
	if { [llength $sections] == 1 } {
	    label $w.f.l$i -text "The dimensioned steel set is:"
	} else {
	    label $w.f.l$i -text "The dimensioned steel sets are:"
	}
	grid $w.f.l$i -row $i -column 1 -sticky w -in $w.f.t -columnspan 2 -pady 5
	foreach j $sections {
	    incr i
	    frame $w.f.f$i
	    label $w.f.f$i.l -text "Steel set:"
	    createtextaslabel $w.f.f$i.e
		
	    button $w.f.f$i.b -image $ProblemTypePriv(phi) -command \
		    "$w.f.f$i.e insert insert f Symbol"

	    GidHelp "$w.f.f$i.b" "This button inserts the diameter symbol in the "\
		    "current cursor position"

	    button $w.f.f$i.b2 -image $ProblemTypePriv(arrow-r) -command \
		    [list EnterNewSteelsSetsFromResults $w._ask $w.f.f$i.e $j]

	    GidHelp "$w.f.f$i.b2" "This button inserts the calculated steel set to the"\
		    " steel sets list"

	    pack $w.f.f$i.l  -side left
	    pack $w.f.f$i.e  -side left -expand 1 -fill x
	    pack $w.f.f$i.b -side left -padx 0
	    pack $w.f.f$i.b2 -side left -padx 2
	    grid $w.f.f$i -row $i -column 1 -sticky ew -in $w.f.t -columnspan 2
	    PutTextinLabelText $w.f.f$i.e $j
	    
	}
    }

    grid $w.f.l -row 1 -column 1 -sticky ew -pady $GUIState::FramesSeparation -columnspan 2
    grid $w.f.t -row 2 -column 1 -sticky ew  -padx 10 -pady 3

    grid columnconf $w.f 1 -weight 1
    grid columnconf $w.f 2 -weight 1

    button $w.ok -text OK -width 10 -command "set tkPriv(rd_button) ok" -und 0
    button $w.cancel -text "View results" -width 10 -command "set tkPriv(rd_button) viewresults" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.ok"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.ok"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-v> "tkButtonInvoke $w.cancel"
    
    grid $w.f -row 1 -column 1 -columnspan 2 -pady $GUIState::FramesSeparation -sticky ewns \
	    -padx 2
    grid $w.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $w.cancel -row 2 -column 2 -pady 2 -sticky w
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    tkwait variable tkPriv(rd_button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }

    if { $tkPriv(rd_button) == "viewresults" } {
	ConcretePrefs::ViewLastHtmlFile $base
    }
}

proc SteelsSetCheck { idx args } {
    global SteelSet
    
    if { $SteelSet($idx) } {
	foreach i $args {
	    catch { $i conf -fg black }
	    catch { $i conf -state normal }
	}
    } else {
	foreach i $args {
	    catch { $i conf -fg grey60 }
	    catch { $i conf -state disabled}
	}
    }
    UpdateSteelSetName
}

proc CreateSteelSetNum { fr name recsec } {
    global SteelSet

    if { ![winfo exists $fr] } { return }

    for { set i 0 } { $i < 3 } { incr i } {
	if { ![winfo exists $fr.c$i] } {
	    if { ![info exists SteelSet($name,check$i)] && $i == 0 } {
		set SteelSet($name,check$i) 1
	    }
	    if { ![info exists SteelSet($name,sep$i)] } {
		set SteelSet($name,sep$i) 10
	    }
	    if { ![info exists SteelSet($name,num$i)] } {
		set SteelSet($name,num$i) 2
	    }
	    checkbutton $fr.c$i -var SteelSet($name,check$i)

	    set values ""
	    for { set j 1 } { $j <= 40 } { incr j } {
		lappend values $j
	    }
	    ComboBox $fr.cb$i -editable 1 -width 3 -textvariable SteelSet($name,num$i) \
		-values $values

#             combobox $fr.cb$i -editable 1 -grab global -width 2 \
#                     -textvariable SteelSet($name,num$i)
#             for { set j 1 } { $j <= 20 } { incr j } {
#                 $fr.cb$i add $j
#             }
	    set tt "tk_optionMenu $fr.om$i SteelSet($name,fi$i) "
	    set nums ""
	    foreach j [info vars ConcretePrefs::steellong*] {
		if { [set $j] } { 
		    regexp {[0-9]*$} $j num
		    append nums "$num "
		}
	    }
	    foreach j [lsort -integer $nums] {
		lappend tt f$j
	    }
	    eval $tt
	    $fr.om$i conf -width 3 -font SymbolFont
	    [$fr.om$i cget -menu] conf -font SymbolFont
	    
	    label $fr.l$i -text 1
	    label $fr.l2$i -text "c/"
	    
	    entry $fr.e$i -width 3 -textvar SteelSet($name,sep$i)


	    label $fr.l3$i -text "cm"

	    trace var SteelSet($name,check$i) w "SteelsSetCheck $name,check$i \
		    $fr.cb$i $fr.om$i $fr.l$i $fr.l2$i $fr.e$i $fr.l3$i"
	    trace var SteelSet($name,num$i) w "UpdateSteelSetName;#"
	    trace var SteelSet($name,fi$i) w "UpdateSteelSetName;#"
	    trace var SteelSet($name,sep$i) w "UpdateSteelSetName;#"
	    # to activate the trace
	    set SteelSet($name,check$i) $SteelSet($name,check$i)
	}

	if { $recsec } {
	    grid $fr.c$i -row $i -column 0 -padx 1 -pady 3 -sticky w
	    grid $fr.cb$i -row $i -column 1 -padx 1 -pady 2 -sticky w
	    grid $fr.om$i -row $i -column 2 -padx 1 -pady 2 -sticky w
	    grid forget $fr.l$i $fr.l2$i $fr.e$i $fr.l3$i
	    grid columnconf $fr 2 -weight 1
	    grid columnconf $fr 5 -weight 0
	} else {
	    grid $fr.c$i -row $i -column 0 -padx 1 -pady 3 -sticky w
	    grid $fr.l$i -row $i -column 1 -padx 1 -pady 2 -sticky w
	    grid $fr.om$i -row $i -column 2 -padx 1 -pady 2 -sticky w
	    grid $fr.l2$i -row $i -column 3 -padx 0 -pady 2 -sticky w
	    grid $fr.e$i -row $i -column 4 -padx 0 -pady 2 -sticky w
	    grid $fr.l3$i -row $i -column 5 -padx 0 -pady 2 -sticky w            
	    grid forget $fr.cb$i
	    grid columnconf $fr 2 -weight 0
	    grid columnconf $fr 5 -weight 1
	}
    }
}

proc CreateSteelSetTrans { fr } {
    global SteelSet

    if { ![info exists SteelSet(trans,check)] } {
	set SteelSet(trans,check) 1
    }
    if { ![info exists SteelSet(trans,sep)] } {
	set SteelSet(trans,sep) 10
    }
    if { ![info exists SteelSet(trans,num)] } {
	set SteelSet(trans,num) 2
    }
    checkbutton $fr.c -var SteelSet(trans,check)

    set values ""
    for { set j 1 } { $j <= 40 } { incr j } {
	lappend values $j
    }
    ComboBox $fr.cb -editable 1 -width 3 -textvariable SteelSet(trans,num) \
	-values $values

#     combobox $fr.cb -editable 1 -grab global -width 2 -textvariable SteelSet(trans,num)
#     for { set j 1 } { $j <= 10 } { incr j } {
#         $fr.cb add $j
#     }
    set tt "tk_optionMenu $fr.om SteelSet(trans,fi) "
    set nums ""
    foreach j [info vars ConcretePrefs::steeltransv*] {
	if { [set $j] } { 
	    regexp {[0-9]*$} $j num
	    append nums "$num "
	}
    }
    foreach j [lsort -integer $nums] {
	lappend tt f$j
    }
    eval $tt
    $fr.om conf -width 3 -font SymbolFont
    [$fr.om cget -menu] conf -font SymbolFont

    label $fr.l -text "c/"

    entry $fr.e -width 3 -textvar SteelSet(trans,sep)

    label $fr.l2 -text "cm"

    trace var SteelSet(trans,check) w "SteelsSetCheck trans,check \
		    $fr.cb $fr.om $fr.l $fr.e $fr.l2"
    trace var SteelSet(trans,num) w "UpdateSteelSetName;#"
    trace var SteelSet(trans,fi) w "UpdateSteelSetName;#"
    trace var SteelSet(trans,sep) w "UpdateSteelSetName;#"

    # to activate the trace
    set SteelSet(trans,check) $SteelSet(trans,check)

    grid $fr.c -row 1 -column 0 -padx 1 -pady 2 -sticky w
    grid $fr.cb -row 1 -column 1 -padx 1 -pady 2 -sticky w
    grid $fr.om -row 1 -column 2 -padx 1 -pady 2 -sticky w
    grid $fr.l -row 1 -column 3 -padx 0 -pady 2 -sticky w
    grid $fr.e -row 1 -column 4 -padx 0 -pady 2 -sticky w
    grid $fr.l2 -row 1 -column 5 -padx 0 -pady 2 -sticky w
    grid columnconf $fr 2 -weight 1
}

proc UpdateSteelSetName {} {
    global SteelSet

    after cancel UpdateSteelSetNameIdle
    after idle UpdateSteelSetNameIdle 
}

proc UpdateSteelSetNameIdle {} {
    global SteelSet

    set update 1
    if { ![info exists SteelSet(autoname)] || !$SteelSet(autoname) } { set update 0 }

    if { [UpdateNameVariable $update] != "" } { return }
    if { !$update } { return }
    if { ![winfo exists $SteelSet(entrytext)] } { return }
    PutTextinLabelText $SteelSet(entrytext) $SteelSet(name)
}

proc UpdateNameVariable { { update 1 } } {
    global SteelSet

    if { ![info exists SteelSet(trans,num)] } { return }
    set PI 3.14159265358979323846
    set name ""
    set SteelSet(areasinfo) ""
    if { $SteelSet(symmetric) } { append name "Symmetric " }
    set isinit 0
    set asval 0.0
    for { set i 0 } { $i < 3 } { incr i } {
	if { !$SteelSet(as,check$i) } { continue }
	if { !$isinit } {
	    append name "As="
	    set isinit 1
	} else { append name "," }
	set finum [string range $SteelSet(as,fi$i) 1 end]
	
	if { $SteelSet(recsec) } {
	    set num $SteelSet(as,num$i)
	    if { [string trim $num] == "" } { set num 0 }
	    if { ![string is integer $num] || $num < 0 } {
		return "error: Bad number in As must be a correct number"
	    }
	    append name "$SteelSet(as,num$i)D$finum"
	    set asval [expr $asval+$num*$PI*($finum/20.0)*($finum/20.0)]
	} else {
	    if { [catch { expr double($SteelSet(as,sep$i)) }] } {
		return "error: Bar separation in steel As must be a correct number"
	    }
	    if { $SteelSet(as,sep$i) <= 0 } {
		return "error: Bar separation in steel As must be a positive number"
	    }
	    append name "1D${finum}c/$SteelSet(as,sep$i)cm"
	    set asval [expr $asval+$PI*($finum/20.0)*($finum/20.0)*100.0/$SteelSet(as,sep$i)]
	}
    }
    if { $isinit } { 
	append name " " 
	if { $SteelSet(recsec) } {
	    append SteelSet(areasinfo) "As=[format %.2f $asval]cm2 "
	} else {
	    append SteelSet(areasinfo) "As=[format %.2f $asval]cm2/m "
	}
    }

    if { !$SteelSet(symmetric) } {
	set isinit 0
	set asval 0.0
	for { set i 0 } { $i < 3 } { incr i } {
	    if { !$SteelSet(asp,check$i) } { continue }
	    if { !$isinit } {
		append name "A's="
		set isinit 1
	    } else { append name "," }
	    set finum [string range $SteelSet(asp,fi$i) 1 end]
	    if { $SteelSet(recsec) } {
		set num $SteelSet(asp,num$i)
		if { [string trim $num] == "" } { set num 0 }
		if { ![string is integer $num] || $num < 0 } {
		    return "error: Bad number in A's must be a correct number"
		}
		append name "$SteelSet(asp,num$i)D$finum"
		set asval [expr $asval+$num*$PI*($finum/20.0)*($finum/20.0)]
	    } else {
		if { [catch { expr double($SteelSet(asp,sep$i))}] } {
		    return "error: Bar separation in steel A's must be a correct number"
		}
		if { $SteelSet(asp,sep$i) <= 0 } {
		    return "error: Bar separation in steel A's must be a positive number"
		}
		append name "1D${finum}c/$SteelSet(asp,sep$i)cm"
		set asval [expr $asval+$PI*($finum/20.0)*($finum/20.0)*100.0/$SteelSet(asp,sep$i)]
	    }
	}
	if { $isinit } { 
	    append name " "
	    if { $SteelSet(recsec) } {
		append SteelSet(areasinfo) "A's=[format %.2f $asval]cm2 "
	    } else {
		append SteelSet(areasinfo) "A's=[format %.2f $asval]cm2/m "
	    }
	}
    }
    set asval 0.0
    if { $SteelSet(trans,check) } {
	set finum [string range $SteelSet(trans,fi) 1 end]
	if { [catch { expr double($SteelSet(trans,sep)) }] } {
	    return "error: Bar separation in steel As,t must be a correct number"
	}
	if { $SteelSet(trans,sep) <= 0 } {
	    return "error: Bar separation in steel As,t must be a positive number"
	}
	append name "At=$SteelSet(trans,num)D${finum}c/$SteelSet(trans,sep)cm"
	set asval [expr $asval+$PI*($finum/20.0)*($finum/20.0)*100.0/$SteelSet(trans,sep)]
	append SteelSet(areasinfo) "At=[format %.2f $asval]cm2/m "
    }
    if { $update } {
	set SteelSet(name) [string trim $name]
    }
    return ""
}

proc UpDateSteelSetVars { text } {
    global SteelSet

    if { [string first Symmetric $text] != -1 } {
	set SteelSet(symmetric) 1
    } else {
	set SteelSet(symmetric) 0
    }
    foreach "name type" "As as A's asp" {
	if { [regexp "$name=(\[^ ]*)" $text {} st] } {
	    regexp {^([^,]+)(,([^,]+))?(,([^,]+))?} $st {} s(0) {} s(1) {} s(2)
	    for { set i 0 } { $i < 3 } { incr i } {
		if { $s($i) != "" } {
		    set SteelSet($type,check$i) 1
		    if { [regexp {^([0-9]+)D([0-9]+)$} $s($i) {} num1 num2] } {
		        set SteelSet($type,num$i) $num1
		        set SteelSet($type,fi$i) "f$num2"
		        set SteelSet(recsec) 1
		    } elseif { [regexp {^([0-9]+)D([0-9]+)c/(.*)cm$} $s($i) {} num1 num2 num3] } {
		        set SteelSet($type,num$i) $num1
		        set SteelSet($type,fi$i) "f$num2"
		        set SteelSet($type,sep$i) $num3
		        set SteelSet(recsec) 0
		    } else {
		        WarnWin "Error decoding Steel set. Unkonwn set: '$text'"
		    }
		} else { set SteelSet($type,check$i) 0 }
	    }
	} else {
	    for { set i 0 } { $i < 3 } { incr i } {
		set SteelSet($type,check$i) 0
	    }
	}
    }
    
    if { [regexp "At=(\[^ ]*)" $text {} st] } {
	if { [regexp {^([0-9]+)D([0-9]+)c/(.*)cm$} $st {} num1 num2 num3] } {
	    set SteelSet(trans,num) $num1
	    set SteelSet(trans,fi) "f$num2"
	    set SteelSet(trans,sep) $num3
	    set steelset(recsec) 0
	} else { set SteelSet(trans,check) 0 }
    } else { set SteelSet(trans,check) 0 }

    #      foreach i [lsort [array names SteelSet]] {
	#          WarnWinText $i--$SteelSet($i)--
	#      }

}

proc PutTextinLabelText { w text { clear yes } } {

    set state [$w cget -state]
    $w conf -state normal
    if { $clear == "yes" } {
	$w del 1.0 end
    }

    while { 1 } {
	set idx [string first D $text]
	if { $idx == -1 } {
	    $w ins end $text
	    break
	} else {
	    $w ins end [string range $text 0 [expr $idx-1]]
	    set text [string range $text [expr $idx+1] end]
	    $w ins end f Symbol
	}
    }
    $w conf -state $state

}

proc ReturnTextFromLabelText { w idx1 idx2 } {

    set text ""
    set disconnect 0

    foreach "key val idx" [$w dump -tag -text $idx1 $idx2] {
	switch $key {
	    text { 
		if { !$disconnect} {
		    append text $val
		}
	    }
	    tagon {
		if { $val == "Symbol" } {
		    append text D
		    set disconnect 1
		}
	    }
	    tagoff {
		if { $val == "Symbol" } {
		    set disconnect 0
		}
	    }
	}
    }
    return [string trim $text]
}

proc createtextaslabel { w } {

    text $w -height 1 -width 10 -wrap none
    $w tag conf Symbol -font SymbolFont
    bind $w <Return> "break"
    bind $w <Down> "break"
    bind $w <KeyPress> {
	if { [regexp {[a-zA-Z]} %A] } {
	    catch {
		if {[%W compare sel.first <= insert]
		&& [%W compare sel.last >= insert]} {
		    %W delete sel.first sel.last
		}
	    }
	    %W insert insert %A ""
	    break 
	}
    }
    return $w
}

proc CreateSteelSet { w  { secname "" } } {
    global tkPriv tcl_platform SteelSet ProblemTypePriv

    catch { destroy $w }
    toplevel $w
    if { $secname == "" } {
	wm title $w "Create steel set"
    } else {
	wm title $w "Edit steel set"
    }
    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    if { $secname != "" } {
	UpDateSteelSetVars $ConcretePrefs::SteelSets($secname)
	if { $ConcretePrefs::SteelSets($secname) == $secname } {
	    set SteelSet(autoname) 1
	} else {
	    set SteelSet(autoname) 0
	}
    } else {
	set SteelSet(autoname) 1
    }
    if { ![info exists SteelSet(recsec)] } {
	set SteelSet(recsec) 1
    }
    if { ![info exists SteelSet(symmetric)] } {
	set SteelSet(symmetric) 0 
    }

    frame $w.f -bd 2 -relief ridge

    frame $w.f.name

    checkbutton $w.f.name.c -text "Name:" -var SteelSet(autoname)
    set SteelSet(entrytext) [createtextaslabel $w.f.name.e]

    GidHelp "$w.f.name.c" "Is this option is checked, name is automatically calculated" \
	    " from selected steel."

    GidHelp "$w.f.name.e" "Name of the Steel set. To change it, uncheck the auto option"

    if { $secname != "" } {
	PutTextinLabelText $w.f.name.e $secname
    }
    
    button $w.f.name.b -image $ProblemTypePriv(phi) -command "\
	    if \$SteelSet(autoname) {WarnWin {Uncheck the auto option to insert the diameter symbol}} \
	    else \"$w.f.name.e insert insert f Symbol\""

    GidHelp "$w.f.name.b" "This button inserts the diameter symbol in the current cursor position"

    grid $w.f.name.c -row 1 -column 1
    grid $w.f.name.e -row 1 -column 2 -sticky ew
    grid $w.f.name.b -row 1 -column 3 -sticky w
    grid columnconf $w.f.name 2 -weight 1

    proc UpdateAutoName { w } {
	global SteelSet
	if { ![winfo exists $w.f.name.e] } { return }
	if {$SteelSet(autoname) } {
	    $w.f.name.e conf -state disabled -fg grey40
	    UpdateSteelSetName
	} else {
	    $w.f.name.e conf -state normal -fg black
	}
    }

    trace var SteelSet(autoname) w "UpdateAutoName $w ;#"

    frame $w.f.left
    
    framelabel $w.f.left.f1 "As"
    CreateSteelSetNum $w.f.left.f1 as $SteelSet(recsec)


    framelabel $w.f.left.f2 "A's"
    CreateSteelSetNum $w.f.left.f2 asp $SteelSet(recsec)

    framelabel $w.f.left.f3 "As,t"
    CreateSteelSetTrans $w.f.left.f3

    grid $w.f.left.f1 -row 1 -column 1 -sticky ewns -pady $GUIState::FramesSeparation
    grid $w.f.left.f2 -row 2 -column 1 -sticky ewns -pady $GUIState::FramesSeparation
    grid $w.f.left.f3 -row 3 -column 1 -sticky ewns -pady $GUIState::FramesSeparation
    grid columnconf $w.f.left 1 -weight 1
    grid rowconf $w.f.left 1 -weight 1
    grid rowconf $w.f.left 2 -weight 1
    grid rowconf $w.f.left 3 -weight 1

    frame $w.f.right -bd 2

    framelabel $w.f.right.f1 "properties"

    checkbutton $w.f.right.f1.c1 -text "Symmetric section" -var SteelSet(symmetric)
    checkbutton $w.f.right.f1.c2 -text "Rectangular section" -var SteelSet(recsec)
    
    GidHelp $w.f.right.f1.c1 "Symmetric section means that the steel will be the same for"\
	    " the four sides of the section. The first steel group should have 2 bars in order "\
	    "to represent the corner bars"

    GidHelp $w.f.right.f1.c2 "In rectangular section, there are given a fixed amount of bars.\n"\
	    "In general, it is given the bar type and the separation"


    trace var SteelSet(recsec) w "\
	    CreateSteelSetNum $w.f.left.f1 as \$SteelSet(recsec) ;\
	    CreateSteelSetNum $w.f.left.f2 asp \$SteelSet(recsec) ;\
	    UpdateSteelSetName ;#"

    grid $w.f.right.f1.c1 -row 1 -column 1 -sticky w -pady 2
    grid $w.f.right.f1.c2 -row 2 -column 1 -sticky w
    grid columnconf $w.f.right.f1 1 -weight 1

    label $w.f.right.l -image $ProblemTypePriv(concrete-section) -bd 2 -relief sunken


    proc UpdateSymmetric { w } {
	global SteelSet ProblemTypePriv
	if { ![winfo exists $w.f.right.l] } { return }
	if { $SteelSet(symmetric) } {
	    $w.f.right.l conf -image $ProblemTypePriv(concrete-section2)
	    grid remove $w.f.left.f2
	} else {
	    $w.f.right.l conf -image $ProblemTypePriv(concrete-section)
	    grid $w.f.left.f2
	}
	UpdateSteelSetName
    }
    trace var SteelSet(symmetric) w "UpdateSymmetric $w ;#"
    
    # to activate trace
    set SteelSet(symmetric) $SteelSet(symmetric)


    grid $w.f.right.f1 -row 1 -column 1 -sticky ew -pady $GUIState::FramesSeparation
    grid $w.f.right.l -row 2 -column 1 -pady 10
    grid columnconf $w.f.right 1 -weight 1
    grid rowconf $w.f.right 1 -weight 1
    grid rowconf $w.f.right 2 -weight 1


    label $w.f.la -textvariable SteelSet(areasinfo)

    grid $w.f.name -row 1 -column 1 -sticky ew -columnspan 2 -pady 3 -padx 5
    grid $w.f.left -row 2 -column 1 -sticky nsew -padx 5
    grid $w.f.right -row 2 -column 2 -sticky nsew -padx 5
    grid $w.f.la -row 3 -column 1 -sticky ew -columnspan 2 
    grid columnconf $w.f 1 -weight 1
    grid columnconf $w.f 2 -weight 1
    grid rowconf $w.f 2 -weight 1


    button $w.ok -text OK -width 5 -command "set tkPriv(css_button) ok" -und 0
    button $w.cancel -text Cancel -width 5 -command "set tkPriv(css_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.cancel"
    bind $w.ok <Return> "tkButtonInvoke $w.ok"
    bind $w <Escape> "tkButtonInvoke $w.cancel"
    bind $w <Alt-o> "tkButtonInvoke $w.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.cancel"
    bind $SteelSet(entrytext) <Return> "tkButtonInvoke $w.ok ; break"

    grid $w.f -row 1 -column 1 -sticky nsew  -columnspan 2
    grid $w.ok -row 2 -column 1 -sticky e -padx 2 -pady 3
    grid $w.cancel -row 2 -column 2 -sticky w -padx 2 -pady 3
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 1 -weight 1

    # to invoke the trace
    set SteelSet(autoname) $SteelSet(autoname)

    wm withdraw $w
    update idletasks

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    WmGidGeom $w +$x+$y
    update
    wm deiconify $w

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    focus $w.ok
    after 100 catch [list "focus -force $w.ok"]

    while 1 {
	tkwait variable tkPriv(css_button)

	if { $tkPriv(css_button) == "ok" } {
	    set name [ReturnTextFromLabelText $w.f.name.e 1.0 end]
	    set retval [UpdateNameVariable]
	    set value $SteelSet(name)
	    if { $name == "" } {
		WarnWin "It is necessary to enter a name for the steel set"
	    } elseif { [string match _* $name] } {
		WarnWin "The name of the steel set cannot begin with '_'"
	    } elseif { $retval != "" } {
		WarnWin $retval
	    } elseif { $secname == "" } {
		set retval [SteelSetsOkDiffNameValue $w.ask $name $value]
		if { $retval != "ok" } {
		    continue
		}
		set retval 0
		set isnew 1
		if { [info exists ConcretePrefs::SteelSets($name)] } {
		    set retval [tk_dialogRAM  $w Warning \
		            "Steel set '$name' already exists. Overwrite?"\
		            warning 0 OK Cancel]
		    set isnew 0
		}
		if { $retval == 0 } {
		    if { $isnew } { lappend ConcretePrefs::SteelSetsNames $name }
		    set ConcretePrefs::SteelSets($name) $value
		    break
		}
	    } elseif { $secname == $name } {
		set retval [SteelSetsOkDiffNameValue $w.ask $name $value]
		if { $retval == "ok" } {
		    set ConcretePrefs::SteelSets($name) $value
		    break
		}
	    } else {
		set retval [SteelSetsOkPrefs $w.ask]
		if { $retval != "ok" } { continue }
		if { !$ConcretePrefs::steelsetsmaintainoldset } {
		    set idx [lsearch $ConcretePrefs::SteelSetsNames $secname]
		    set ConcretePrefs::SteelSetsNames [lreplace $ConcretePrefs::SteelSetsNames \
		            $idx $idx]
		    unset ConcretePrefs::SteelSets($secname)
		}
		if { !$ConcretePrefs::steelsetscreatenewset } { continue }
		set retval 0
		set isnew 1
		if { [info exists ConcretePrefs::SteelSets($name)] } {
		    set retval [tk_dialogRAM  $w Warning \
		            "Steel set '$name' already exists. Overwrite?"\
		            warning 0 OK Cancel]
		    set isnew 0
		}
		if { $retval == 0 } {
		    if { $isnew } { lappend ConcretePrefs::SteelSetsNames $name }
		    set ConcretePrefs::SteelSets($name) $value
		    break
		}
	    }
	} else { break }
    }

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }
    unset SteelSet
}

proc WaitMessage { f text } {

    frame $f.waitplease -bd 2 -relief raised
    label $f.waitplease.l -text $text
    pack $f.waitplease.l -expand 1
    update idletasks
    set width [expr [winfo reqwidth $f.waitplease.l]*1.2]
    set height [expr [winfo reqheight $f.waitplease.l]*4]
    place $f.waitplease -relx .5 -rely .5  \
	    -width $width -height $height -anchor center
    [winfo toplevel $f] conf -cursor watch
    $f conf -cursor watch
    . conf -cursor watch
    update
    after 500
}

proc EndWaitMessage { f } {
    $f conf -cursor ""
    [winfo toplevel $f] conf -cursor ""
    destroy $f.waitplease
}


namespace eval ConcretePrefs {
    variable ramdir
    variable elem ""
    variable pointelem ""
    variable steeltype B500-S
    variable concretetype HA-25
    variable defaultwidthycm 30
    variable defaultwidthzcm 30
    variable atmosphere IIb
    variable subatmosphere None
    variable execution normal
    variable coveringflag 0 covering Automatic
    variable m1coveringflag 0 m1covering Automatic
    variable m2coveringflag 0 m2covering Automatic
    variable separationsflag 0 separations Automatic
    variable steellong6 0 steellong8 0 steellong10 1 steellong12 1 steellong14 1
    variable steellong16 1 steellong20 0 steellong25 0 steellong32 0 steellong40 0
    variable steeltransv6 1 steeltransv8 1 steeltransv10 0 steeltransv12 0
    variable alpha 1.0
    variable BeamLengthFlag 0 BeamLength Automatic
    variable xpsense 1
    variable saveprefsonproblemtype 0
    variable SteelSets
    variable SteelSetsNames ""
    variable steelsetsmaintainoldset 0
    variable steelsetscreatenewset 1
    variable lasthtmlfile ""
    variable elemuse Automatic
    variable elemname ""
    variable Strengths
    variable SymbolsAsImages 0
    variable reporttitle ""
    variable reportfilename ""
    variable reportdescription ""
    variable reportlastimgcaption ""
    variable reportexportdir ""
    variable varstosave "steeltype concretetype defaultwidthycm defaultwidthzcm atmosphere \
	    subatmosphere execution coveringflag covering m1coveringflag m1covering m2coveringflag \
	    m2covering separationsflag separations steellong* \
	    steeltransv* alpha BeamLengthFlag BeamLength xpsense saveprefsonproblemtype \
	    SteelSets SteelSetsNames \
	    steelsetsmaintainoldset steelsetscreatenewset lasthtmlfile elem pointelem \
	    elemuse elemname Strengths \
	    SymbolsAsImages reporttitle reportfilename reportdescription reportlastimgcaption \
	    reportexportdir"
    variable varstosaveglobal "steeltype concretetype defaultwidthycm defaultwidthzcm atmosphere \
	    subatmosphere execution coveringflag covering m1coveringflag m1covering m2coveringflag \
	    m2covering separationsflag separations steellong* \
	    steeltransv* alpha BeamLengthFlag BeamLength saveprefsonproblemtype SteelSets \
	    SteelSetsNames steelsetsmaintainoldset steelsetscreatenewset Strengths SymbolsAsImages \
	    reporttitle reportdescription reportexportdir"

    proc init { dir } {
	variable coveringflag 
	variable m1coveringflag 
	variable m2coveringflag 
	variable separationsflag
	variable BeamLengthFlag
	variable ramdir
	variable Strengths
	set ramdir $dir
	# to activate the trace var
	set coveringflag $coveringflag
	set m1coveringflag $m1coveringflag
	set m2coveringflag $m2coveringflag
	set separationsflag $separationsflag
	set BeamLengthFlag $BeamLengthFlag
	set Strengths(servicefactor) 1.5

	variable SteelSets
	variable SteelSetsNames
	catch { unset SteelSets }
	set SteelSetsNames ""
	Read
    }
    proc SelectElement { fr } {
	global WarnLineVar
	
	frame $fr.__f -bd 2 -relief ridge -width [expr [winfo width $fr]-0] \
		-height [expr [winfo height $fr]-0]
	label $fr.__f.l -text "Pick one point in a beam or shell element"
	button $fr.__f.b -text Cancel -command "ConcretePrefs::EndSelectionElement $fr.__f 1 \
		WarnLineVar {} w"
	pack $fr.__f.l $fr.__f.b -side left -padx 3 -pady 3
	pack propagate $fr.__f 0
	focus $fr.__f.b
	place $fr.__f -in $fr -relx 0 -rely 0 -anchor nw -bordermode outside

	gidmakecurrent
	.central.s process escape escape escape escape utilities id PointInElement
	trace var WarnLineVar w "ConcretePrefs::EndSelectionElement $fr.__f 0"
    }
    proc EndSelectionElement { frame iscancel name1 name2 op } {
	variable elem
	variable pointelem
	global WarnLineVar
	upvar #0 $name1 cad
	
	destroy $frame

	if { $cad == "Leaving" } {
	    trace vdelete WarnLineVar w "ConcretePrefs::EndSelectionElement $frame 0"
	    return
	}

	if { $iscancel } {
	    .central.s process escape
	} elseif {[regexp {from element[ ]+([0-9]+)} $cad {} elem] } {
	    #nothing
	} elseif { ![regexp \
		{Point number[ ]*([^.]*)[^=]*=([^, ]*)[^=]*=([^, ]*)[^=]*=([^, ]*)} \
		$cad trash pntnum xval yval zval]} { 
	    return 
	} else {
	    set pointelem $xval,$yval,$zval
	    trace vdelete WarnLineVar w "ConcretePrefs::EndSelectionElement $frame 0"
	    .central.s process escape
	}
    }
    proc ViewPoint {} {
	variable pointelem
	if { $pointelem == "" } {
	    WarnLine "There is no point to view. Select firt one"
	    return
	}
	.central.s process escape escape escape escape Utilities SignalEntities \
		Nodes FNoJoin $pointelem

    }
    proc GiveTempDir {} {
	global env

	if { [info exists env(TEMP)] } {
	    return $env(TEMP)
	}
	if { [info exists env(TMP)] } {
	    return $env(TMP)
	}
	if { [info exists env(WINDIR)] && [file isdir [file join $env(WINDIR) temp]] } {
	    return [file join $env(WINDIR) temp]
	}
	if { [file isdir /tmp] } {
	    return /tmp
	}
	return ""
    }
    proc GiveProjectDir {} {
	set aa [.central.s info Project]
	set ProjectName [lindex $aa 1]
	if { [file extension $ProjectName] == ".gid" } {
	    set ProjectName [file root $ProjectName]
	}
	
	if { $ProjectName == "UNNAMED" } {
	    return ""
	}
	
	set directory $ProjectName.gid
	if { [file pathtype $directory] == "relative" } {
	    set directory [file join [pwd] $directory]
	}
	return $directory
    }
    proc GiveReportName {} {

	set aa [.central.s info Project]
	set ProjectName [lindex $aa 1]
	if { [file extension $ProjectName] == ".gid" } {
	    set ProjectName [file root $ProjectName]
	}
	
	if { $ProjectName == "UNNAMED" } {
	    set dir [GiveTempDir]
	    if { $dir != "" } {
		return [file join $dir tmp_report.html]
	    }
	    return ""
	}
	
	set directory $ProjectName.gid
	if { [file pathtype $directory] == "relative" } {
	    set directory [file join [pwd] $directory]
	}
	return [file join $directory [file tail $ProjectName].html]
    }
    proc GivePrefsDataForDimension {} {
    	variable steeltype
	variable atmosphere
	variable subatmosphere
	variable execution
	variable covering
	variable m1covering
	variable m2covering
	variable separations
	variable alpha
	variable BeamLength
	variable elemuse
	variable elemname
	variable SymbolsAsImages

	set retval ""
	append retval "problem type concrete dimension\n"

	foreach i [info vars ::ConcretePrefs::steel*] {
	    variable [namespace tail $i]
	}
	if { $SymbolsAsImages } {
	    append retval "SymbolsAsImages\n"
	}
	append retval "steeltype $steeltype\n"
	append retval "atmosphere $atmosphere\n"
	append retval "subatmosphere $subatmosphere\n"
	append retval "execution $execution\n"
	append retval "covering $covering\n"
	append retval "m1covering $m1covering\n"
	append retval "m2covering $m2covering\n"
	append retval "separations $separations\n"
	append retval "alpha $alpha\n"
	append retval "BeamLength $BeamLength\n"
	set ipos [lsearch [.central.s info gendata] Gravity_X*]
	append retval "gravity [lindex [.central.s info gendata] [expr $ipos+1]] \
		[lindex [.central.s info gendata] [expr $ipos+3]] \
		[lindex [.central.s info gendata] [expr $ipos+5]]\n"

	append retval "ElemUse $elemuse\n"
	append retval "ElemName $elemname\n"

	set buf ""
	foreach i [info vars steellong*] {
	    regexp {[0-9]+} $i num
	    if { [set $i] } { lappend buf $num }
	}
	if { $buf == "" } {
	    error "error: there is no longitudinal steel selected"
	}
	append retval "steellong [lsort -integer $buf]\n"

	set buf ""
	foreach i [info vars steeltransv*] {
	    regexp {[0-9]+} $i num
	    if { [set $i] } { lappend buf $num }
	}
	if { $buf == "" } {
	    error "error: there is no transversal steel selected\n"
	}
	append retval "steeltransv [lsort -integer $buf]\n"
	return $retval
    }

    proc Dimension { w what { secname "" } } {
	variable elem
	variable pointelem
	variable concretetype
	variable defaultwidthycm
	variable defaultwidthzcm
	variable xpsense
	variable lasthtmlfile ""

	if { $what == "elem" } {
	    if { [catch { expr int($elem) } err] } {
		WarnWin "Before dimensioning, it is necessary to select a point in a mesh element"
		return
	    }
	    set type ""
	    foreach i "Rectangular_Section Section Concrete_rec_section Shell Concrete_Shell \
		    Anisotropic_Shell" {
		set retval [lindex [.central.s info conditions $i mesh $elem] 0]
		if { [lindex $retval 0] != 0 } {
		    set type $i
		    break
		}
	    }
	} else { set type "strengths" }
	switch $type {
	    Section {
		set retval [AskForWidth $w.askforwidth]
		if { $retval != "ok" } { return }
		set widthY [expr $defaultwidthycm*1e-2]
		set widthZ [expr $defaultwidthzcm*1e-2]
		set concrete_res $concretetype
	    }
	    Concrete_rec_section {
		set units [lindex $retval 4]
		set fac [ConvertUnitsToSI $units m]
		if { $fac == "" } { return }
		set widthY [expr $fac*[lindex $retval 5]]
		set widthZ [expr $fac*[lindex $retval 6]]
		set concrete_res [lindex $retval 7]
	    }
	    Rectangular_Section {
		set units [lindex $retval 4]
		set fac [ConvertUnitsToSI $units m]
		if { $fac == "" } { return }
		set widthY [expr $fac*[lindex $retval 5]]
		set widthZ [expr $fac*[lindex $retval 6]]
		set concrete_res $concretetype
	    }
	    Concrete_Shell {
		set units [lindex $retval 4]
		set fac [ConvertUnitsToSI $units m]
		if { $fac == "" } { return }
		set widthY 1.0
		set widthZ [expr $fac*[lindex $retval 5]]
		set concrete_res [lindex $retval 6]
	    }
	    Shell {
		set units [lindex $retval 4]
		set fac [ConvertUnitsToSI $units m]
		if { $fac == "" } { return }
		set widthY 1.0
		set widthZ [expr $fac*[lindex $retval 5]]
		set concrete_res $concretetype
	    }
	    strengths {
		set widthY [expr $defaultwidthycm*1e-2]
		set widthZ [expr $defaultwidthzcm*1e-2]
		set concrete_res $concretetype
	    }
	    "" {
		WarnWin "Problems obtaining elements properties"
		return
	    }
	    default {
		WarnWin "Properties type '$type' not implemented"
		return
	    }
	}

	catch { unset Results }
	if { $what == "elem" } {
	    set val [.central.s info list_entities Elements $elem]
	    if { [lindex $val 0] == "Bad" } {
		WarnWin "Problems obtaining elements results"
		return
	    }
	    
	    set nodes [lindex $val 3]
	    set elemtype [lindex $val 4]
	    set nres [lindex [lindex $val 8] 1]
	    set allres [lindex $val 12]
	    
	    set validres { {"B Axial Force" N} {BDisplacements m} {Torsor Nm} {"Y' Momentus" Nm} \
		    {"Y' Shear" N} {"Z' Momentus" Nm} {"Z' Shear" N} {"Local Axes Beam" ""} \
		    {"Axial Force" N/m} {Momentus N } {Shear N/m} {"Local Axes" ""} }
	    
	    set curr [.central.s info postprocess get cur_analisis]
	    set alla [.central.s info postprocess get all_analisis]

	    set ipos [lsearch $alla Max_Min]
	    if { $ipos != -1 } {
		set alla [lreplace $alla $ipos $ipos]
	    }

	    set ipos [lsearch [.central.s info gendata] Results_units*]
	    set units [lindex [.central.s info gendata] [expr $ipos+1]]

	    foreach k $alla {
		.central.s process escape escape escape escape Results AnalysisSel $k \
		        escape escape
		set val [.central.s info list_entities Elements $elem]
		set pos [lsearch $val More]
		incr pos
		incr pos
		set allres [lindex $val $pos]
		foreach i $allres {
		    set resname [lindex $i 0]
		    regexp {^(.*)[ ]+[(][^\)]*[)]$} $resname {} resname
		    set found 0
		    set fac ""
		    foreach r $validres {
		        if { $resname == [lindex $r 0] } {
		            if { [lindex $r 1] == "" } {
		                set fac 1.0
		            } else {
		                set fac [ConvertUnitsToSI $units [lindex $r 1]]
		            }
		            set found 1
		            break
		        }
		    }
		    if { !$found } { continue }
		    if { $fac == "" } { return }
		    set type [lindex $i 1]
		    set Results($k,$resname) ""
		    switch $type {
		        Beams {
		            foreach j [lindex $i 3] {
		                foreach l [lindex $j 3] {
		                    lappend Results($k,$resname) [expr $fac*$l]
		                }
		            }                                     
		        }                                     
		        Beams1 {                             
		            foreach j [lindex $i 3] {             
		                lappend Results($k,$resname) [lindex $j 0]
		                lappend Results($k,$resname) [lindex $j 1]
		                lappend Results($k,$resname) [lindex $j 2]
		            }
		        }
		        Triangles {
		            set inum 1
		            foreach j [lindex $i 3] {
		                append Results($k,$resname) " GP$inum"
		                foreach l $j {
		                    lappend Results($k,$resname) [expr $fac*$l]
		                }
		                incr inum
		            }
		        }
		        Triangles1 {
		            foreach j [lindex $i 3] {
		                lappend Results($k,$resname) [lindex $j 0]
		                lappend Results($k,$resname) [lindex $j 1]
		                lappend Results($k,$resname) [lindex $j 2]
		            }
		        }
		    }
		}
	    }
	    .central.s process escape escape escape escape Results AnalysisSel $curr \
		    escape escape
	} else {
	    variable Strengths
	    set elemtype ""
	    set nres 1
	    set alla 1
	    set "Results(Estatic,B Axial Force)" 0.0
	    set "Results(Estatic,Y' Momentus)" 0.0
	    set "Results(Estatic,Z' Momentus)" 0.0
	    set "Results(Estatic,Z' Shear)" 0.0
	    catch { set "Results(Estatic,B Axial Force)" [expr $Strengths(axial)*1e3] }
	    catch { set "Results(Estatic,Y' Momentus)" [expr $Strengths(M1)*1e3] }
	    catch { set "Results(Estatic,Z' Momentus)" [expr $Strengths(M2)*1e3] }
	    catch { set "Results(Estatic,Z' Shear)" [expr $Strengths(shear)*1e3] }

	}
	set dir [GiveProjectDir]
	if { $dir == "" } { 
	    set dir [GiveTempDir]
	    if { $dir == "" } { 
		WarnWin "Could not find temporal directory. Save project to continue"
		return
	    }
	}
	if { [catch {
	    set fin [open [file join $dir tmp_concrete.dat] w]
	}]} {
	    WarnWin "error: filed to write to file '[file join $dir tmp_concrete.dat]'"
	    return
	}
	variable steeltype
	variable atmosphere
	variable subatmosphere
	variable execution
	variable covering
	variable m1covering
	variable m2covering
	variable separations
	variable alpha
	variable BeamLength
	variable elemuse
	variable elemname
	variable SymbolsAsImages

	
	puts $fin "problem type concrete dimension"

	foreach i [info vars ::ConcretePrefs::steel*] {
	    variable [namespace tail $i]
	}
	if { $SymbolsAsImages } {
	    puts $fin "SymbolsAsImages"
	}
	puts $fin "steeltype $steeltype"
	puts $fin "atmosphere $atmosphere"
	puts $fin "subatmosphere $subatmosphere"
	puts $fin "execution $execution"
	puts $fin "covering $covering"
	puts $fin "m1covering $m1covering"
	puts $fin "m2covering $m2covering"
	puts $fin "separations $separations"
	puts $fin "alpha $alpha"
	puts $fin "BeamLength $BeamLength"
	set ipos [lsearch [.central.s info gendata] Gravity_X*]
	puts $fin "gravity [lindex [.central.s info gendata] [expr $ipos+1]] \
		[lindex [.central.s info gendata] [expr $ipos+3]] \
		[lindex [.central.s info gendata] [expr $ipos+5]]"

	  puts $fin "ElemUse $elemuse"
	  puts $fin "ElemName $elemname"

	set buf ""
	foreach i [info vars steellong*] {
	    regexp {[0-9]+} $i num
	    if { [set $i] } { lappend buf $num }
	}
	if { $buf == "" } {
	    WarnWin "error: there is no longitudinal steel selected"
	    return
	}
	puts $fin "steellong [lsort -integer $buf]"

	set buf ""
	foreach i [info vars steeltransv*] {
	    regexp {[0-9]+} $i num
	    if { [set $i] } { lappend buf $num }
	}
	if { $buf == "" } {
	    WarnWin "error: there is no transversal steel selected"
	    return
	}
	puts $fin "steeltransv [lsort -integer $buf]"

	if { $what == "elem" } {
	    set ipos [lsearch [.central.s info gendata] Mesh_units*]
	    set meshunits [lindex [.central.s info gendata] [expr $ipos+1]]
	    set fac [ConvertLengthsUnitsToSI $meshunits]
	    foreach i $nodes {
		set coords [lindex [.central.s info Coordinates $i] 0]
		set var "Node $i"
		foreach j $coords {
		    lappend var "[expr $fac*$j]"
		}
		puts $fin $var
	    }
	    puts $fin "element $elem $nodes"
	    regexp {([^,]+),([^,]+),([^,]+)} $pointelem {} x y z
	    set pointelemL "[expr $fac*$x],[expr $fac*$y],[expr $fac*$z]"
	    puts $fin "PointInElement $pointelemL"
	} else {
	    if { [catch { expr double($Strengths(servicefactor)) }] } {
		set Strengths(servicefactor) 0.0
	    }
	    if { $Strengths(servicefactor) < 0.0 } {
		WarnWin "error: Service factor must be positive"
		return
	    }
	    if { $Strengths(servicefactor) > 0.0 && $Strengths(servicefactor) < 1.0 } {
		WarnWin "error: Service factor must be bigger or equal than 1.0"
		return
	    }
	    puts $fin "ServiceFactor $Strengths(servicefactor)"
	}
	puts $fin "widthY $widthY"
	puts $fin "widthZ $widthZ"
	puts $fin "concrete_res $concrete_res"
	if { $elemtype != "Linear" && $elemtype != ""} {
	    if { $xpsense } {
		puts $fin "shell_element xpsense"
	    } else {
		puts $fin "shell_element ypsense"
	    }
	}

	if { $secname != "" } {
	    variable SteelSets
	    puts $fin "Section $SteelSets($secname)"
	}

	proc sortres { e1 e2 } {
	    regexp {([^,]+),([^,]+)} $e1 {} el1 res1
	    regexp {([^,]+),([^,]+)} $e2 {} el2 res2
	    if { $el1 < $el2 } { return -1 }
	    if { $el1 > $el2 } { return 1 }
	    if { $res1 < $res2 } { return -1 }
	    if { $res1 > $res2 } { return 1 }
	    return 0
	}

	set results [lsort -command sortres [array names Results]]
	puts $fin "NumResults $nres"
	puts $fin "NumLC [llength $alla]"

	set val [lsearch [.central.s info gendata] Activate_Load_Cases*]
	incr val
	set justone 0
	if { [lindex [.central.s info gendata] $val] == "none" } {
	    set justone 1
	}
	set val [lsearch [.central.s info gendata] Combined_load_cases*]
	incr val

	if { $what != "elem" } { set justone 1 }

	set lastLC ""
	foreach i $results {
	    regexp {([^,]+),([^,]+)} $i {} LC ip
	    if { $LC != $lastLC } {
		if { $justone } {
		    set isstrengths 1
		} else {
		    set val2 [lsearch [lindex [.central.s info gendata] $val] $LC]
		    if { $val2 == -1 } {
		        WarnWin "error with loadcases"
		        return
		    }
		    incr val2
		    set isstrengths [lindex [lindex [.central.s info gendata] $val] $val2]
		}
		if { $lastLC != "" } {
		    puts $fin "end loadcase"
		}
		puts $fin "loadcase $LC $isstrengths"
	    }
	    puts $fin "$ip $Results($i)"
	    set lastLC $LC
	}
	puts $fin "end loadcase"
	puts $fin "end element"
	close $fin

	variable ramdir
	set exec [file join $ramdir rambshell]

	WaitMessage $w "Dimensioning. Wait please..."
	if { [catch {
	    exec $exec -c [file join $dir tmp_concrete.dat]
	}]} {
	    EndWaitMessage $w
	    WarnWin "There have been problems in the execution"
	    return
	}
	EndWaitMessage $w
	set lasthtmlfile [file join $dir tmp_concrete.html]

	if { $secname == "" } {
	    ResultDimension $w.res $w [file join $dir tmp_concrete.res] dimension
	} else {
	    ResultDimension $w.res $w [file join $dir tmp_concrete.res] check
	}
    }
    proc SaveFile { file { isglobal } } {
	variable varstosave
	variable varstosaveglobal

	if { [catch {
	    set fout [open $file w]
	} err] } {
	    WarnWin [concat "It is not possible to save data in file '$file'. "\
		    "Check your writing permissions"]
	    return -1
	}

	if { !$isglobal } {
	    set vars $varstosave
	} else {
	    set vars $varstosaveglobal
	}

	foreach i $vars {
	    foreach j [info vars ::ConcretePrefs::$i] {
		set var [namespace tail $j]
		variable $var
		if { [array exists $var] } {
		    puts $fout "begin array $var"
		    foreach k [array names $var] {
		        puts $fout "   [list $k [set ${var}($k)]]"
		    }
		    puts $fout "end array"
		} elseif { [info exists $var] } {
		    puts $fout [list $var [set $var]]
		}
	    }
	}
	close $fout
	return 0
    }
    proc SaveLocal {} {
	set dir [GiveProjectDir]
	if { $dir != "" } {
	    set file [file join $dir concrete.prefs]
	    SaveFile $file 0
	}
    }
    proc CreateAndGiveDefaultsFile {} {

	set dir [file dirname [GiveGidDefaultsFile]]
	if { $::tcl_platform(platform) == "windows" } {
	    set dir [file join $dir RamSeries]
	} else {
	    set dir [file join $dir .ramseries]
	}
	if { ![file isdir $dir] } {
	    catch { file mkdir $dir }
	}
	return $dir
    }
    proc Save {} {
	variable saveprefsonproblemtype 
	variable ramdir 
	
	set retval -1 ; set retval2 -1;

	if { $saveprefsonproblemtype } {
	    set file [file join [CreateAndGiveDefaultsFile] concrete.prefs]
	    set retval [SaveFile $file 1]
	}
	if { !$saveprefsonproblemtype || $retval } {
	    set dir [GiveProjectDir]
	    if { $dir != "" } {
		set file [file join $dir concrete.prefs]
		set retval2 [SaveFile $file 0]
	    }
	}

	if { $saveprefsonproblemtype && $retval } {
	    if { !$retval2 } {
		WarnWin [concat "It is not possible to save data in Rambshell. "\
		        "Check your writing permissions. Data has been saved locally"]
	    } else {
		WarnWin [concat "It is not possible to save data in Rambshell. "\
		        "Check your writing permissions. Data has NOT been saved"]
	    }
	}
	if { !$saveprefsonproblemtype && $retval2 } {
	    WarnWin [concat "It is not possible to save data in local project. "\
		    "Check your writing permissions and disk space. Data has NOT been saved"]
	}
    }
    proc ReadFile { file } {
	if { [catch {
	    set fin [open $file r]
	} ] } { return }
	set array ""
	while { ![eof $fin] } {
	    gets $fin aa
	    set name [lindex $aa 0]
	    set value [lindex $aa 1]
	    if { $name == "begin" && $value == "array" } {
		set array [lindex $aa 2]
		variable $array
		catch { unset $array }
	    } elseif { $name == "end" && $value == "array" } {
		set array ""
	    } elseif { $array != "" } {
		set [set array]($name) $value
	    } else {
		variable $name
		set $name $value
	    }
	}
	close $fin
    }
    proc Read {} {
	variable ramdir
	set file [file join [CreateAndGiveDefaultsFile] concrete.prefs]
	ReadFile $file
	set dir [GiveProjectDir]
	if { $dir != "" } {
	    set file [file join $dir concrete.prefs]
	    ReadFile $file
	}
    }
    proc ViewLastHtmlFile { base } {
	variable lasthtmlfile
	variable ramdir

	if { $lasthtmlfile == "" } {
	    WarnWin "There are no results to view"
	    return
	}
	if { ![file readable $lasthtmlfile] } {
	    WarnWin "Results cannot be viewed"
	    return
	}
	set top [winfo toplevel $base]
	set x [expr [winfo x $top]+[winfo width $top]/2-350]
	set y [expr [winfo y $top]+[winfo height $top]/2-300]
	set geom 650x600+${x}+$y
	
	#ViewHelpFor . $lasthtmlfile
	HelpWindowRam -geomhelp $geom -tt -title Results -win .gid.results $lasthtmlfile
#          set exec [file join $ramdir HelpViewer]
#          exec $exec -geomhelp $geom -tt -title Results $lasthtmlfile &
    }
    proc ViewReportHtmlFile { base } {
	variable reportfilename
	variable ramdir

	if { $reportfilename == "" } {
	    WarnWin "There is no report to view. Create it first with button 'Create'"
	    return
	}

	set reportfilename [GiveReportName]

	if { ![file readable $reportfilename] } {
	    WarnWin "Report cannot be viewed (check problems in file: '$reportfilename')"
	    return
	}
	set top [winfo toplevel $base]
	set x [expr [winfo x $top]+[winfo width $top]/2-350]
	set y [expr [winfo y $top]+[winfo height $top]/2-300]
	set geom 650x600+${x}+$y
	HelpWindowRam -geomhelp $geom -tt -title Report -win .gid.report $reportfilename

#          set exec [file join $ramdir HelpViewer]
#          exec $exec -geomhelp $geom -tt -title Report $reportfilename &
    }
    proc ViewHelpFor { base file } {
	variable ramdir

#          set top [winfo toplevel $base]
#          set x [expr [winfo x $top]+[winfo width $top]/2-350]
#          set y [expr [winfo y $top]+[winfo height $top]/2-300]
#          set geom 650x600+${x}+$y
#          set exec [file join $ramdir HelpViewer]
#          exec $exec -geomhelp $geom -tt [file join $ramdir $file] &
	HelpWindow CUSTOM_HELP_FILE [file join $ramdir scripts $file]
    }
    proc AddCurrentViewToReport { base } {
	variable reportfilename
	variable reportlastimgcaption

	if { $reportfilename == "" } {
	    WarnWin "There is no report where to add current view. Create it first with button 'Create'"
	    return
	}
	
	set reportfilename [GiveReportName]

	if { ![file writable $reportfilename] } {
	    WarnWin "Report cannot be modified (check problems in file: '$reportfilename')"
	    return
	}
	set dir [file join [file dirname $reportfilename] images]
	set i 1
	if { ![file exists $dir] } {
	    file mkdir $dir
	}

	set ext .jpg

	while { [file exists [file join $dir view$i$ext]] } { incr i }
	set img [file join $dir view$i$ext]
	set shortimg [file join images view$i$ext]
	.central.s process Hardcopy JPEG \"$img\" escape

	file rename $reportfilename $reportfilename.tmp
	set fin [open $reportfilename.tmp r]
	set fout [open $reportfilename w]
	while { ![eof $fin] } {
	    gets $fin aa
	    if { [regexp {</body>} $aa] } {
		puts $fout "<a name=\"image\"></a>"
		puts $fout "<br clear=all>"
		puts $fout "<table>\n<tr><td>"
		puts $fout "<img src=\"$shortimg\" align=\"center\">"
		puts $fout "</tr>\n<tr><td><center>$reportlastimgcaption</center></tr></table>"
	    } 
	    puts $fout $aa
	}
	close $fin
	close $fout
	file delete $reportfilename.tmp
	WarnWinReport "Image inserted OK" $base
    }
    proc AddCurrentResultsToReport { base } {
	variable reportfilename
	variable lasthtmlfile

	if { $reportfilename == "" } {
	    WarnWin [concat "There is no report where to add current results. "\
		    "Create it first with button 'Create'"]
	    return
	}

	set reportfilename [GiveReportName]

	if { ![file writable $reportfilename] } {
	    WarnWin "Report cannot be modified (check problems in file: '$reportfilename')"
	    return
	}
	if { ![file readable $lasthtmlfile] } {
	    WarnWin "There are no results to add. Create them with 'Dimension' or 'Check'"
	    return
	}
	set imgtodir [file join [file dirname $reportfilename] images]
	set fromdir [file dirname $lasthtmlfile]
	set i 1
	if { ![file exists $imgtodir] } {
	    file mkdir $imgtodir
	}
	file rename $reportfilename $reportfilename.tmp
	set fin [open $reportfilename.tmp r]
	set finres [open $lasthtmlfile r]
	set fout [open $reportfilename w]
	while { ![eof $fin] } {
	    gets $fin aa
	    if { [regexp {</body>} $aa] } {
		puts $fout "<a name=\"results\"></a>"
		puts $fout "<br clear=all><hr width=90% align=center>"
		set isinit 0
		while { ![eof $finres] } {
		    gets $finres bb
		    if { [regexp {<body} $bb] } {
		        set isinit 1
		    } elseif { !$isinit } { continue }
		    if { [regexp {</body} $bb] } { break }
		    set ini 0
		    while { [regexp -indices "<img\[ ]+src=\"(\[^\"]+)\"" [string range $bb $ini end] \
		            {} imgidx] } {
		        set idx1 [expr $ini+[lindex $imgidx 0]]
		        set idx2 [expr $ini+[lindex $imgidx 1]]
		        set imgname [string range $bb $idx1 $idx2]
		        if { [regexp {^sym-} [file tail $imgname]] } {
		            if { ![file exists  [file join $imgtodir [file tail $imgname]]] } {
		                file copy [file join $fromdir $imgname] $imgtodir
		            }
		            set newname [file tail $imgname]
		        } else {
		            regexp {^[^0-9.]+} [file tail $imgname] shortname
		            set ext [file ext $imgname]
		            set i 1
		            while { [file exists [file join $imgtodir $shortname$i$ext]] } { incr i }
		            set newname $shortname$i$ext
		            set imgtoname [file join $imgtodir $newname]
		            file copy [file join $fromdir $imgname] $imgtoname
		        }
		        set bb [join [list [string range $bb 0 [expr $idx1-1]] \
		                [file join images $newname] [string range $bb \
		                [expr $idx2+1] end]] ""]
		        set ini $idx2
		    }
		    puts $fout $bb
		}
	    } 
	    puts $fout $aa
	}
	close $fin
	close $fout
	close $finres
	file delete $reportfilename.tmp
	WarnWinReport "Results inserted OK" $base
    }
    proc DeleteLastFromReport { base what } {
	variable reportfilename
	variable lasthtmlfile

	if { $reportfilename == "" } {
	    WarnWin "There is no report where to delete last $what. Create it first with button 'Create'"
	    return
	}

	set reportfilename [GiveReportName]

	if { ![file writable $reportfilename] } {
	    WarnWin "Report cannot be modified (check problems in file: '$reportfilename')"
	    return
	}

	set fin [open $reportfilename r]
	set nline 1
	set linebegin ""
	set lineend ""
	while { ![eof $fin] } {
	    gets $fin aa
	    if { $what == "results" } {
		if { [regexp "<a\[ ]+name=\"results\"" $aa] } {
		    set linebegin $nline
		    set lineend ""
		} elseif { [regexp "<a\[ ]+name=\"image\"" $aa] } {
		    if { $linebegin != "" && $lineend == "" } {
		        set lineend [expr $nline -1]
		    }
		}
	    } elseif { $what == "image" } {
		if { [regexp "<a\[ ]+name=\"image\"" $aa] } {
		    set linebegin $nline
		    set lineend ""
		} elseif { [regexp "<a\[ ]+name=\"results\"" $aa] } {
		    if { $linebegin != "" && $lineend == "" } {
		        set lineend [expr $nline -1]
		    }
		}
	    }
	    if { [regexp {</body>} $aa] } {
		if { $linebegin != "" && $lineend == "" } {
		    set lineend [expr $nline -1]
		}
		break
	    }
	    incr nline
	}
	close $fin
	if { $linebegin == "" } {
	    WarnWin "error: There is no $what to delete in the report"
	    return
	}
	set retval [tk_dialogRAM  $base._tmp Warning "Are you sure to delete last $what from report?" \
		warning 0 OK Cancel]
	if { $retval == 1 } { return }

	file rename $reportfilename $reportfilename.tmp
	
	set fin [open $reportfilename.tmp r]
	set fout [open $reportfilename w]

	set dir [file dirname $reportfilename]
	set nline 1
	while { ![eof $fin] } {
	    gets $fin aa
	    if { $nline < $linebegin || $nline > $lineend } {
		puts $fout $aa
	    } else {
		set ini 0
		while { [regexp -indices "<img\[ ]+src=\"(\[^\"]+)\"" [string range $aa $ini end] \
		        {} imgidx] } {
		    set idx1 [expr $ini+[lindex $imgidx 0]]
		    set idx2 [expr $ini+[lindex $imgidx 1]]
		    set imgname [file join $dir [string range $aa $idx1 $idx2]]
		    if { [regexp {^sym-} [file tail $imgname]] } {
		        # nothing
		    } elseif { [file exists $imgname] } {
		        file delete $imgname
		    }
		    set ini $idx2
		}
	    }
	    incr nline
	}
	close $fin
	close $fout
	file delete $reportfilename.tmp
	WarnWinReport "Last $what deleted OK from report" $base
    }

    # what can be: HTML or Word
    proc ExportReport { base what } {
	variable reportfilename
	variable reportexportdir

	if { $reportfilename == "" } {
	    WarnWin "There is no report to export. Create it first with button 'Create'"
	    return
	}

	set reportfilename [GiveReportName]
	switch $what {
	    HTML {
		set types {
		    {{HTML file}      {.html .htm}   }
		    {{All Files}       *             }
		}
		set ext ".html"
		set initial [file tail $reportfilename]
	    }
	    Word {
		set types {
		    {{Word file}      {.doc}         }
		    {{All Files}       *             }
		}
		set ext ".doc"
		set initial [file root [file tail $reportfilename]].doc
	    }
	}

	if { ![file readable $reportfilename] } {
	    WarnWin "Report cannot be read (check problems in file: '$reportfilename')"
	    return
	}
	if { ![file isdir $reportexportdir] } {
	    set reportexportdir [pwd]
	}
	global tk_strictMotif
	set tk_strictMotif_save $tk_strictMotif
	set tk_strictMotif 0
	set filename [tk_getSaveFile -defaultextension $ext -filetypes $types \
		          -initialdir $reportexportdir -initialfile $initial -parent $base \
		          -title "Export report"]
	set tk_strictMotif $tk_strictMotif_save
	if { $filename == "" } { return }
	set reportexportdir [file dirname $filename]
	if { [file ext $filename] == ".html" || [file ext $filename] == ".htm" } {
	    set imgdir [file join $reportexportdir images]
	    if { [file exists $imgdir] } {
		set retval [tk_dialogRAM $base._tmp Warning \
		                "Are you sure to delete directory '$imgdir' and all its contents?"\
		                warning 0 OK Cancel]
		if { $retval == 1 } { return }
		if { [catch {
		    file delete -force $imgdir
		} err] } {
		    WarnWin "error: Could not delete directory '$imgdir' ($err)"
		    return
		}
	    }
	    
	    if { [catch {
		file copy -force [file join [file dirname $reportfilename] images] \
		    $imgdir
		file copy -force $reportfilename $filename
	    } err] } {
		WarnWin [concat "Problems exporting report to '$filename' ($err). "\
		             "Check write permissions and disk space"]
		return
	    }
	} elseif { [file ext $filename] == ".doc" || [file ext $filename] == ".rtf" } {
	    rtf:HTML2RTF $reportfilename $filename "Ram Series"
	} else {
	    WarnWin "Unknown extension for file '$tofile'"
	    return
	}
	if { $::tcl_platform(platform) == "windows" } {
	    set text "Report exported OK to file '$filename' Do you want to visualize it?"
	    set retval [tk_messageBox -default no -icon question -message $text \
		            -parent $base -title "Report exported" -type yesno]
	    if { $retval == "yes" } {
		eval exec [auto_execok start] [list [file native $filename]] &
	    }
	} else {
	    WarnWin "Report exported OK to file '$filename'"
	}
    }
     proc ExportDXF { base } {
	variable reportexportdir
	variable reportfilename

	set dir [GiveProjectDir]
	set dxffile [file join $dir tmp_concrete.dxf]
	if { ![file readable $dxffile] } {
	    WarnWin "There is no DXF file to export. Use buttons 'Dimension' or 'Check' to create"
	    return
	}

	if { ![file isdir $reportexportdir] } {
	    set reportexportdir [pwd]
	}
	global tk_strictMotif
	set tk_strictMotif_save $tk_strictMotif
	set tk_strictMotif 0
	set initialfile ""
	if { $reportfilename != "" } {
	    set initialfile [file join [file root [file tail $reportfilename]].dxf]
	}
	set filename [tk_getSaveFile -defaultextension .dxf -filetypes {{"DXF files" ".dxf .DXF"}} \
		-initialdir $reportexportdir -initialfile $initialfile -parent $base \
		-title "Export DXF"]
	set tk_strictMotif $tk_strictMotif_save
	if { $filename == "" } { return }
	set reportexportdir [file dirname $filename]

	if { [catch {
	    file copy -force $dxffile $filename
	} err] } {
	    WarnWin [concat "Problems exporting DXF file to '$filename' ($err). "\
		    "Check write permissions and disk space"]
	    return
	}
	WarnWin "DXF exported OK to file '$filename'"
    }
    proc CreateFileForReport { filename } {

	if { [catch {
	    set fout [open $filename w]
	} err] } {
	    error "error: could not write file '$filename' ($err)"
	}
	#WARNING
	fconfigure $fout -encoding binary
	return $fout
    }
    proc CloseFileForReport { fout } {

	close $fout
    }
    proc putshtml { fout text } {
	variable SymbolsAsImages

	if { $SymbolsAsImages } {
	    # whe have only symbol phi
	    #regsub -all {<sym>f</sym>} $text {<img src="images/sym-f.gif">} text
	    regsub -all {<sym>f</sym>} $text {<img src="images/sym-f.jpg">} text
	}

	regsub -all {<sym>} $text "<tt><font face=\"Symbol\">" text
	regsub -all {</sym>} $text {</font></tt>} text

	set maps {
	      "&aacute;"
	      "&eacute;"
	      "&iacute;"
	      "&oacute;"
	      "&uacute;"
	      "&ntilde;"
	      "&middot;"
	}
	set text [string map $maps $text]
	
	if { $fout != "" } {
	    puts $fout $text
	} else {
	    rtf:writehtml "" $text
	}
    }
    proc CreateReport { base } {
	variable ramdir
	variable reporttitle
	variable reportfilename
	variable reportdescription
	variable SymbolsAsImages

	set name [GiveReportName]

	if { $name == "" } {
	    WarnWin "Could not find temp directory. Save current project"
	    return
	}

	if { [file exists $name] } {
	    set retval [tk_dialogRAM  $base._temp Warning "File '$name' exists, overwrite?" \
		    warning 0 OK Cancel]
	    if { $retval == 1 } { return }
	}
	if { [catch {
	    set fout [CreateFileForReport $name]
	} err] } {
	    WarnWin "error: could not write file '$name' ($err)"
	    return
	}

	if { $SymbolsAsImages } {
	    set dir [file join [GiveProjectDir] images]
	    if { [file isdir $dir] } {
		file delete -force $dir
	    }
	    file mkdir $dir
	    file copy [file join $ramdir images sym-f.jpg] $dir
	}
	set title [string trim $reporttitle]
	if { $title == "" } {
	    set title "--ttulo vaco--"
	}
	putshtml $fout "<html>\n<head>\n<title>$title</title>\n</head>\n<body bgcolor=\"#FFFFFF\">"
	putshtml $fout "<h1>$title</h1>"

	putshtml $fout "<p>$reportdescription</p>"

	set info [.central.s info list_entities status]
	regexp "name:\[ ]*(\[^\n]*)\n" $info {} projectname
	regexp "nodes:\[ ]*(\[^\n]*)\n" $info {} nnodes
	set lines 0
	regexp "lines:\[ ]*(\[^\n]*)\n" $info {} lines
	set triangles 0
	regexp "triangles:\[ ]*(\[^\n]*)\n" $info {} triangles

	putshtml $fout "<table border=1 align=left>"
	putshtml $fout "<tr><td>Proyecto<td>$projectname</tr>"
	putshtml $fout "<tr><td>Num. nodos<td>$nnodes</tr>"
	if { $lines > 0 } {
	    putshtml $fout "<tr><td>Num. barras<td>$lines</tr>"
	}
	if { $triangles > 0 } {
	    putshtml $fout "<tr><td>Num. elm. lmina<td>$triangles</tr>"
	}
	putshtml $fout "</table>"
	putshtml $fout "<br clear=all>"

	set val [lsearch [.central.s info gendata] Activate_Load_Cases*]
	incr val
	set justone 0
	if { [lindex [.central.s info gendata] $val] == "none" } {
	    set justone 1
	}
	if { $justone } {
	    putshtml $fout "<h2>Hay una nica hiptesis de carga</h2>"
	} else {
	    set val [lsearch [.central.s info gendata] Combined_load_cases*]
	    incr val
	    set lcases [lrange [lindex [.central.s info gendata] $val] 2 end]
	    set lsimple ""
	    foreach "LC isstrengths combs" $lcases {
		foreach "fac namel" [split $combs ,] {
		    if { [lsearch $lsimple $namel] == -1 } {
		        lappend lsimple $namel
		    }
		}
	    }
	    #putshtml $fout "<br clear=all>"
	    putshtml $fout "<h2>Hiptesis de carga</h2>"
	    putshtml $fout "<table border=1 align=left>"
	    putshtml $fout "<tr><td><b>hip. combinada</b><td><b>est. lim. lt.</b>"
	    foreach i $lsimple {
		putshtml $fout "<td><b>$i</b>"
	    }
	    putshtml $fout "</tr>"

	    foreach "lc isstrengths combs" $lcases {
		putshtml $fout "<tr><td>$lc"
		if { $isstrengths } {
		    putshtml $fout "<td>s"
		} else { putshtml $fout "<td>no" }
		set facs ""
		foreach i $lsimple { lappend facs 0.0 }
		foreach "fac namel" [split $combs ,] {
		    set idx [lsearch $lsimple $namel]
		    if { $idx != -1 } {
		        set facs [lreplace $facs $idx $idx $fac]
		    }
		}
		foreach i $facs { putshtml $fout "<td>[format %g $i]" }
	    }
	    putshtml $fout "</tr>"
	    putshtml $fout "</table>"
	}

	putshtml $fout "</body>\n</html>"
	CloseFileForReport $fout
	set reportfilename $name

	WarnWinReport "Created report '$name'" $base
    }
}


namespace eval GUIState {
    variable mainwin
    variable FontHeight
    variable FramesSeparation
}

proc OpenGUI { { w .gui } } {
    global tcl_platform 

#      if { $tcl_platform(platform) != "windows" } {
#          option add *Scrollbar*Width 10
#          option add *Scrollbar*BorderWidth 1
#          option add *Button*BorderWidth 1
#      } else {
#          option add *Button*padY -2
#      }
#      option add *Menu*TearOff 0
  
#      if { $tcl_platform(platform) != "windows" } {
#          option add *background AntiqueWhite3
#          option add *Button*background bisque3
#          option add *Menu*background bisque3
#          option add *Button*foreground black
#          option add *Entry*background thistle
#          option add *DisabledForeground grey60
#          option add *HighlightBackground AntiqueWhite3
#      }

    if { [info procs InitWindow] != "" } {
	InitWindow $w "Dimension/check armed concrete" PostDimConcreteWindowGeom DimensionConcrete
    } else {
	toplevel $w
	wm title $w "Dimension/check armed concrete"
    }

    set GUIState::mainwin $w
    
    wm withdraw $w

    frame $w.buts

    button $w.buts.ok -text OK -command "ConcretePrefs::SaveLocal ; destroy $w" -under 0 -width 6
    button $w.buts.cancel -text Cancel -command "destroy $w" -under 0 -width 6
    button $w.buts.save -text "Save..." -command "SavePrefs $w.prefs" -under 0 -width 6
    button $w.buts.hp -image [imagefromram questionarrow.gif] -command "PickHelp $w.buts.hp" \
	    -highlightthickness 0
      pack $w.buts.ok $w.buts.cancel $w.buts.save $w.buts.hp -side left \
	      -padx 1 -ipadx 0 -pady 5

    GidHelp "$w.buts.ok" "This button closes the window and stores all settings with the "\
	    "current project"
    GidHelp "$w.buts.cancel" "This button closes the window and discards all settings"
    GidHelp "$w.buts.save" "This button permmits to store the settings either in the current "\
	    "project or in the rambshell database. In this later case, new projects will have "\
	    "acces to these settings."

    #focus $w.buts.close
    bind $w <Alt-o>  "tkButtonInvoke $w.buts.ok ; break"
    bind $w <Alt-c>  "tkButtonInvoke $w.buts.cancel ; break"
    bind $w <Alt-s>  "tkButtonInvoke $w.buts.save ; break"
    #bind $w <Escape>  "tkButtonInvoke $w.buts.cancel ; break"

    frame $w.f
    grid $w.buts -row 1 -column 0
    grid $w.f -row 0 -column 0 -sticky nsew
    grid rowconfigure $w.f 0 -weight 1
    grid columnconfigure $w.f 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    set all_qq "Dimension {User def} Materials Settings Report"
 
    set func "optionbar $w.f.c"
    foreach qq $all_qq {
	set pref [string tolower $qq]
	regsub -all { } $pref {_} pref
	set holder [frame $w.f.$pref]
	CreateFrame_$qq $holder
	lappend func [list $qq $holder [list ChangeToPrefTypeCON $w $qq]]
    }
    eval $func

    pack $w.f.c -fill both -expand yes

    update idletasks

    set x [expr [winfo screenwidth $w]/2-[winfo reqwidth $w]/2]
    set y [expr [winfo screenheight $w]/2-[winfo reqheight $w]/2]
    if { [info procs InitWindow] == "" } {
	wm geom $w +$x+$y
    }
    wm deicon $w
}

proc DimensionConcrete { dir } {
    global tcl_platform ProblemTypePriv

#      source "optionbar.tcl"
#      source "combo.tcl"
#      source "sortedlist.tcl"

#      if { [catch {
#      #encoding system iso8859-1
#      } err ] } {
#          WarnWin $err--[encoding system]---[encoding names]
#      }
    set size 12
    if { [lsearch [font names] NormalFont] != -1 } {
	set size [font configure NormalFont -size]
    }
    if { [lsearch [font names] BoldFont] == -1 } {
	if { $tcl_platform(platform) == "windows"} {
	    font create BoldFont -family "MS Sans Serif" -size 8
	} else {
	    font create BoldFont -family {new century schoolbook} -size 12
	}
#          option add *font BoldFont
	option add *Button*BorderWidth 1
    }

    if { [lsearch [font names] SmallFont] == -1 } {
	if { $tcl_platform(platform) == "windows"} {
	    font create SmallFont -family "MS Sans Serif" -size [expr $size-2]
	} else {
	    font create SmallFont -family {new century schoolbook} -size [expr $size-2]
	}
    } elseif { [font conf SmallFont -size] >= [font conf NormalFont -size]} {
	set size [font conf NormalFont -size]
	set family [font conf NormalFont -family]
	font conf SmallFont -family $family -size [expr $size-2]
    }
    if { [lsearch [font names] SymbolFont] == -1 } {
	if { $tcl_platform(platform) == "windows"} {
	    font create SymbolFont -family Symbol -size 8
	} else {
	    font create SymbolFont -family Symbol -size 12
	}
#          option add *font BoldFont
	option add *Button*BorderWidth 1
    }

    foreach i "phi arrow-r folder openfold steelset concrete-section concrete-section2" {
	if { ![info exists ProblemTypePriv($i)] } {
	    set ProblemTypePriv($i) [image create photo \
		    -file [file join $ProblemTypePriv(imagesdir) $i.gif]]
	}
    }

    set GUIState::FontHeight [font metric BoldFont -linespace ]
    set GUIState::FramesSeparation [expr $GUIState::FontHeight/2+1]

    OpenGUI .concretedimension
    # just for testing
    if { $tcl_platform(platform) == "windows"} {
	bind all <Control-x><Control-l> {
	    set err [catch {
		source {c:/Users/ramsan/barres/rambshell.gid/scripts/Concrete.tcl}
		 source {c:/Users/ramsan/barres/rambshell.gid/scripts/rtflib.tcl}
		 source {c:/Users/ramsan/barres/rambshell.gid/scripts/HelpViewer.tcl}
		 source {c:/Users/ramsan/barres/rambshell.gid/scripts/rambshell-2.tcl}
		foreach i [array names ProblemTypePriv image-*] {
		    unset ProblemTypePriv($i)
		}
	    } errstring]
	    if { !$err } {
		WarnWin "Concrete code reload"
	    } else {
		WarnWin "Concrete code reload error ($err--$::errorInfo)"
	    }
		
	}
    } else {
	bind all <Control-x><Control-l> "source {/home/ramsan/barres/rambshell.gid/Concrete.tcl}"
    }
    ConcretePrefs::init $dir
}

#source HelpViewer.tcl
#  wm withdraw .
#  set tk_strictMotif 1



