

namespace eval DrawSections {
    variable isactive ""
    variable ipn_object
    variable points
    variable beamcolor [list .8 .8 .8]
}


#--------------------------------------------------------------------------------
# NOT USED BY NOW UNTIL...

proc DrawSections::Activate {} {
    variable isactive

    set isactive [drawopengl register DrawSections::draw]
}

proc DrawSections::DeActivate {} {
    variable isactive

    if { $isactive != "" } {
	drawopengl unregister $isactive
	set isactive ""
    }
}

proc DrawSections::draw {} {
    variable points
    variable beamcolor
    variable ipn_object

    CreateObjects

    set width [winfo width .gid.central.s]
    set height [winfo height .gid.central.s]

    set bmargin 15
    set bwidth 160
    set bheight 120

    drawopengl draw -pushattrib depthbufferbit -pushattrib lightingbit -pushattrib viewportbit \
    -disable lighting -disable depthtest \
    -matrixmode projection \
    -pushmatrix -loadidentity -viewport [list [expr {$width-$bwidth-$bmargin}] $bmargin \
	$bwidth $bheight] \
    -ortho [list 0 1 1 0 -1 1] \
    -matrixmode modelview -pushmatrix -loadidentity \
    -color {0 0 0} -begin lineloop -vertex {0 0} -vertex {1 0} -vertex {1 1} -vertex {0 1} -end \
    -color $beamcolor -begin quads -vertex {0.1 0.1} -vertex {.1 .2} -vertex {.2 .2} -vertex {.2 .1} \
    -end -color {0 0 0} \
    -begin lineloop -vertex {0.1 0.1} -vertex {.1 .2} -vertex {.2 .2} -vertex {.2 .1} -end \
    -color {0 0 0} -rasterpos {.25 .2}
    drawopengl drawtext "Section"
    drawopengl draw -matrixmode projection -popmatrix -matrixmode modelview -popmatrix \
	-popattrib -popattrib -popattrib

    set onlayers ""
    foreach i [.central.s info Layers] {
	if { [lindex [lindex [.central.s info Layers $i] 0] 0]== 1 } {
	    lappend onlayers $i
	}
    }

    if { [catch {.central.s info Conditions Section}]} { return }


    catch { unset points }

    foreach cond [list Rectangular_Section Section Concrete_rec_section Steel_section] {
	foreach i [.central.s info Conditions -localaxesmat $cond geometry] {
	    set num [lindex $i 1]
	    set mat [lindex $i 3]
	    set list_ent [.central.s info list_entities Lines $num]
	    regexp {LAYER:\s*(\S+)} $list_ent {} lay
	    if { [lsearch $onlayers $lay] == -1 } { continue }
	    regexp {Points:\s+([0-9]+)\s+([0-9]+)} $list_ent {} P1 P2
	    foreach j "P1 P2" {
		if { ![info exists points([set $j])] } {
		    set list_pnt [.central.s info list_entities Points [set $j]]
		    regexp {Coord:\s+([^\n]+)} $list_pnt {} coords
		    set points([set $j]) $coords
		}
	    }
	    if { [llength $mat] == 1 } {
		set mat [math::BeamTransfMatrix [list $P1 $P2] points]
	    }
	    set length [math::module [math::VectorDiff $points($P1) $points($P2)]]
	    drawopengl draw -pushmatrix -translate $points($P1) -multmatrix \
		[math::RotMat2OpenGLmat $mat] -scale [list $length .1 .1] -color $beamcolor \
		-begin quads -call $ipn_object -end -popmatrix
	    
	}
    }
}

#   NOT USED UNTIL HERE
#--------------------------------------------------------------------------------

proc DrawSections::drawSectionCond { condname geommesh num values } {
    variable beamcolor
    variable ipn_object

    CreateObjects

    switch $geommesh {
	GEOMETRYUSE {
	    if { [catch {.central.s info list_entities Lines $num} list_ent] } { return 0 }
	    if { [regexp {^\s*STLINE} $list_ent] } {
		set type stline
	    } else { set type nurbline } ;# arc assimilated to nurb
	    
	    regexp {Points:\s+([0-9]+)\s+([0-9]+)} $list_ent {} P1 P2
	    foreach j "P1 P2" {
		set list_pnt [.central.s info list_entities Points [set $j]]
		regexp {Coord:\s+([^\n]+)} $list_pnt {} coords
		set points([set $j]) $coords
	    }
	}
	MESHUSE {
	    if { [lindex [.gid.central.s info Project] 4] == "MESHUSE" } {
		set list_ent [.central.s info list_entities Elements $num]
		set type stline
		regexp {([0-9]+)\s+([0-9]+)\s*$} $list_ent {} P1 P2
		foreach j "P1 P2" {
		    set list_pnt [.central.s info list_entities Nodes [set $j]]
		    regexp {\S+\s+\S+\s+\S+\s*$} $list_pnt coords
		    set points([set $j]) $coords
		}
	    } else {
		set list_ent [.central.s info list_entities Elements $num]
		set type stline
		foreach "P1 P2" [lindex $list_ent 3] break
		foreach j "P1 P2" {
		    set list_pnt [.central.s info list_entities Nodes [set $j]]
		    set points([set $j]) [lindex $list_pnt 3]
		}
	    }
        }
	default { return 0 }
    }
    switch $condname {
	Rectangular_Section { set color [list .5 0 0] }
	Section { set color [list .5 .5 .5] }
	Concrete_rec_section { set color [list .5 .5 .15] }
	Steel_section { set color [list .8 .8 .8] }

    }
    
    set delta .1
    for { set t $delta } { $t <= 1.0+.5*$delta } { set t [expr {$t+$delta}] } {
	if { $type != "stline" } {
	    set points($P2) [split [.central.s info nurbs line $num 0 $t] ,]
	}
	set mat [lindex $values 0]
	if { [llength $mat] == 1 } {
	    set mat [math::BeamTransfMatrix [list $P1 $P2] points]
	} else {
	    set mat [math::CorrectBeamTransfMatrix [list $P1 $P2] points $mat] 
	}
	set length [math::module [math::VectorDiff $points($P1) $points($P2)]]
	drawopengl draw -pushmatrix -translate $points($P1) -multmatrix \
	    [math::RotMat2OpenGLmat $mat] -scale [list $length .2 .2] -color $color \
	    -begin quads -call $ipn_object -end -popmatrix
	if { $type == "stline" } { break }
	set points($P1) $points($P2)
    }
    # return 1 means no draw simbol after
    return 1
}

proc DrawSections::drawLoadCond { condname geommesh num values } {
    variable beamcolor
    variable load_object

    CreateObjects

    switch $geommesh {
	GEOMETRYUSE {
	    if { [catch {.central.s info list_entities Lines $num} list_ent] } { return 0 }
	    if { [regexp {^\s*STLINE} $list_ent] } {
		set type stline
	    } else { set type nurbline } ;# arc assimilated to nurb
	    
	    regexp {Points:\s+([0-9]+)\s+([0-9]+)} $list_ent {} P1 P2
	    foreach j "P1 P2" {
		set list_pnt [.central.s info list_entities Points [set $j]]
		regexp {Coord:\s+([^\n]+)} $list_pnt {} coords
		set points([set $j]) $coords
	    }
	}
	MESHUSE {
	    if { [lindex [.gid.central.s info Project] 4] == "MESHUSE" } {
		set list_ent [.central.s info list_entities Elements $num]
		set type stline
		regexp {([0-9]+)\s+([0-9]+)\s*$} $list_ent {} P1 P2
		foreach j "P1 P2" {
		    set list_pnt [.central.s info list_entities Nodes [set $j]]
		    regexp {\S+\s+\S+\s+\S+\s*$} $list_pnt coords
		    set points([set $j]) $coords
		}
	    } else {
		set list_ent [.central.s info list_entities Elements $num]
		set type stline
		foreach "P1 P2" [lindex $list_ent 3] break
		foreach j "P1 P2" {
		    set list_pnt [.central.s info list_entities Nodes [set $j]]
		    set points([set $j]) [lindex $list_pnt 3]
		}
	    }
        }
	default { return 0 }
    }
    switch $condname {
	Global_Beam_Load {
	    set yvec [math::Normalizevector [lrange $values 1 3]]
	    set color [list .5 0 0]
	}
	Global_Projected_Beam_Load {
	    set yvec [math::Normalizevector [lrange $values 1 3]]
	    set color [list .5 .5 .5]
	}
	default { return 0 }
    }

    set delta .1
    for { set t $delta } { $t <= 1.0+.5*$delta } { set t [expr {$t+$delta}] } {
	if { $type != "stline" } {
	    set points($P2) [split [.central.s info nurbs line $num 0 $t] ,]
	}
	
	set mat [lindex $values 0]
	if { [llength $mat] == 1 } {
	    set mat [math::BeamTransfMatrix [list $P1 $P2] points]
	}
	
	set xvec [math::GetAxeInMatrix $mat x]
	set mat [math::SetAxeInMatrix $mat y $yvec]
	set zvec [math::VectorProd $xvec $yvec]
	if { [math::module $zvec] < 1e-10 } {
	    set zvex [list 0 0 1]
	}
	set mat [math::SetAxeInMatrix $mat z $zvec]
	
	set length [math::module [math::VectorDiff $points($P1) $points($P2)]]
	drawopengl draw -pushmatrix -translate $points($P1) -multmatrix \
	    [math::RotMat2OpenGLmat $mat] -scale [list $length 1 1] -color $color \
	    -begin lines -call $load_object -end -popmatrix
	if { $type == "stline" } { break }
	set points($P1) $points($P2)
    }

    # return 1 means no draw simbol after
    return 1
}

proc DrawSections::CreateObjects {} {
    variable ipn_object
    variable load_object

    if { ![info exists ipn_object] } {
	set ipn_object [drawopengl draw -genlists 1]
	drawopengl draw -newlist $ipn_object compile  \
	-normal {0 0 1} \
	-vertex {1 1 1}     -vertex {0 1 1}     -vertex {0 -1 1}    -vertex {1 -1 1} \
	-normal {0 -1 0} \
	-vertex {1 -1 1}    -vertex {0 -1 1}    -vertex {0 -1 .9}   -vertex {1 -1 .9} \
	-normal {0 0 -1} \
	-vertex {1 -1 .9}   -vertex {0 -1 .9}   -vertex {0 -.1 .9}  -vertex {1 -.1 .9} \
	-normal {0 -1 0} \
	-vertex {1 -.1 .9}  -vertex {0 -.1 .9}  -vertex {0 -.1 -.9} -vertex {1 -.1 -.9} \
	-normal {0 0 1} \
	-vertex {1 -.1 -.9} -vertex {0 -.1 -.9} -vertex {0 -1 -.9}  -vertex {1 -1 -.9} \
	-normal {0 -1 0} \
	-vertex {1 -1 -.9}  -vertex {0 -1 -.9}  -vertex {0 -1 -1}   -vertex {1 -1 -1} \
	-normal {0 0 -1} \
	-vertex {1 -1 -1}   -vertex {0 -1 -1}   -vertex {0 1 -1}    -vertex {1 1 -1} \
	-normal {0 1 0} \
	-vertex {1 1 -1}    -vertex {0 1 -1}    -vertex {0 1 -.9}   -vertex {1 1 -.9} \
	-normal {0 0 1} \
	-vertex {1 1 -.9}   -vertex {0 1 -.9}   -vertex {0 .1 -.9}  -vertex {1 .1 -.9} \
	-normal {0 1 0} \
	-vertex {1 .1 -.9}  -vertex {0 .1 -.9}  -vertex {0 .1 .9}   -vertex {1 .1 .9} \
	-normal {0 0 -1} \
	-vertex {1 .1 .9}   -vertex {0 .1 .9}   -vertex {0 1 .9}    -vertex {1 1 .9} \
	-normal {0 1 0} \
	-vertex {1 1 .9}    -vertex {0 1 .9}    -vertex {0 1 1}     -vertex {1 1 1} \
	\
	-normal {1 0 0} \
	-vertex {0 1 1}     -vertex {0 -1 1}    -vertex {0 -1 .9}   -vertex {0 1 .9} \
	-vertex {0 .1 .9}   -vertex {0 -.1 .9}  -vertex {0 -.1 -.9} -vertex {0 .1 -.9} \
	-vertex {0 1 -1}    -vertex {0 1 -.9}   -vertex {0 -1 -.9}  -vertex {0 -1 -1} \
	\
	-normal {-1 0 0} \
	-vertex {1 1 1}     -vertex {1 1 .9}    -vertex {1 -1 .9}   -vertex {1 -1 1}  \
	-vertex {1 .1 .9}   -vertex {1 .1 -.9}  -vertex {1 -.1 -.9} -vertex {1 -.1 .9} \
	-vertex {1 1 -1}    -vertex {1 -1 -1}   -vertex {1 -1 -.9}  -vertex {1 1 -.9} \
	-endlist
    }

    if { ![info exists load_object] } {
	set load_object [drawopengl draw -genlists 1]
	set comm "drawopengl draw -newlist $load_object compile "
	for { set i 0 } { $i < 5 } { incr i } {
	    set x [expr {$i/4.0}]
	    append comm "-vertex {$x -1 0} -vertex {$x 0 0} "
	    append comm "-vertex {[expr {$x-.025}] -.2 0} -vertex {$x 0 0} "
	    append comm "-vertex {[expr {$x+.025}] -.2 0} -vertex {$x 0 0} "
	}
	append comm "-vertex {0 -1 0} -vertex {1 -1 0} "
	append comm "-endlist"
	eval $comm
    }
}